Package: forcats
Title: Tools for Working with Categorical Variables (Factors)
Version: 1.0.1
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Helpers for reordering factor levels (including moving
    specified levels to front, ordering by first appearance, reversing,
    and randomly shuffling), and tools for modifying factor levels
    (including collapsing rare levels into other, 'anonymising', and
    manually 'recoding').
License: MIT + file LICENSE
URL: https://forcats.tidyverse.org/,
        https://github.com/tidyverse/forcats
BugReports: https://github.com/tidyverse/forcats/issues
Depends: R (>= 4.1)
Imports: cli (>= 3.4.0), glue, lifecycle, magrittr, rlang (>= 1.0.0),
        tibble
Suggests: covr, dplyr, ggplot2, knitr, readr, rmarkdown, testthat (>=
        3.0.0), withr
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-09-24 17:08:21 UTC; hadleywickham
Author: Hadley Wickham [aut, cre],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2025-09-25 05:10:20 UTC
Built: R 4.6.0; ; 2026-04-28 22:36:15 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as_factor,character)
S3method(as_factor,factor)
S3method(as_factor,logical)
S3method(as_factor,numeric)
export("%>%")
export(as_factor)
export(fct)
export(fct_anon)
export(fct_c)
export(fct_collapse)
export(fct_count)
export(fct_cross)
export(fct_drop)
export(fct_expand)
export(fct_explicit_na)
export(fct_infreq)
export(fct_inorder)
export(fct_inseq)
export(fct_lump)
export(fct_lump_lowfreq)
export(fct_lump_min)
export(fct_lump_n)
export(fct_lump_prop)
export(fct_match)
export(fct_na_level_to_value)
export(fct_na_value_to_level)
export(fct_other)
export(fct_recode)
export(fct_relabel)
export(fct_relevel)
export(fct_reorder)
export(fct_reorder2)
export(fct_rev)
export(fct_shift)
export(fct_shuffle)
export(fct_unify)
export(fct_unique)
export(first2)
export(last2)
export(lvls_expand)
export(lvls_reorder)
export(lvls_revalue)
export(lvls_union)
import(rlang)
importFrom(glue,glue)
importFrom(lifecycle,deprecated)
importFrom(magrittr,"%>%")
importFrom(stats,median)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# forcats <img src='man/figures/logo.png' align="right" height="139" />

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/forcats)](https://cran.r-project.org/package=forcats)
[![R-CMD-check](https://github.com/tidyverse/forcats/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/tidyverse/forcats/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/tidyverse/forcats/graph/badge.svg)](https://app.codecov.io/gh/tidyverse/forcats)
<!-- badges: end -->

## Overview

R uses **factors** to handle categorical variables, variables that have
a fixed and known set of possible values. Factors are also helpful for
reordering character vectors to improve display. The goal of the forcats
package is to provide a suite of tools that solve common problems with
factors, including changing the order of levels or the values. Some
examples include:

- `fct_reorder()`: Reordering a factor by another variable.
- `fct_infreq()`: Reordering a factor by the frequency of values.
- `fct_relevel()`: Changing the order of a factor by hand.
- `fct_lump()`: Collapsing the least/most frequent values of a factor
  into “other”.

You can learn more about each of these in `vignette("forcats")`. If
you’re new to factors, the best place to start is the [chapter on
factors](https://r4ds.hadley.nz/factors.html) in R for Data Science.

## Installation

    # The easiest way to get forcats is to install the whole tidyverse:
    install.packages("tidyverse")

    # Alternatively, install just forcats:
    install.packages("forcats")

    # Or the the development version from GitHub:
    # install.packages("pak")
    pak::pak("tidyverse/forcats")

## Cheatsheet

<a href="https://raw.githubusercontent.com/rstudio/cheatsheets/main/factors.pdf"><img src="https://github.com/rstudio/cheatsheets/raw/main/pngs/thumbnails/forcats-cheatsheet-thumbs.png" width="320" height="252"/></a>

## Getting started

forcats is part of the core tidyverse, so you can load it with
`library(tidyverse)` or `library(forcats)`.

``` r
library(forcats)
library(dplyr)
library(ggplot2)
```

``` r
starwars |> 
  filter(!is.na(species)) |>
  count(species, sort = TRUE)
#> # A tibble: 37 × 2
#>    species      n
#>    <chr>    <int>
#>  1 Human       35
#>  2 Droid        6
#>  3 Gungan       3
#>  4 Kaminoan     2
#>  5 Mirialan     2
#>  6 Twi'lek      2
#>  7 Wookiee      2
#>  8 Zabrak       2
#>  9 Aleena       1
#> 10 Besalisk     1
#> # ℹ 27 more rows
```

``` r
starwars |>
  filter(!is.na(species)) |>
  mutate(species = fct_lump(species, n = 3)) |>
  count(species)
#> # A tibble: 4 × 2
#>   species     n
#>   <fct>   <int>
#> 1 Droid       6
#> 2 Gungan      3
#> 3 Human      35
#> 4 Other      39
```

``` r
ggplot(starwars, aes(x = eye_color)) + 
  geom_bar() + 
  coord_flip()
```

![](man/figures/README-unordered-plot-1.png)<!-- -->

``` r
starwars |>
  mutate(eye_color = fct_infreq(eye_color)) |>
  ggplot(aes(x = eye_color)) + 
  geom_bar() + 
  coord_flip()
```

![](man/figures/README-ordered-plot-1.png)<!-- -->

## More resources

For a history of factors, I recommend [*stringsAsFactors: An
unauthorized
biography*](https://simplystats.github.io/2015/07/24/stringsasfactors-an-unauthorized-biography/)
by Roger Peng and [*stringsAsFactors =
\<sigh\>*](https://notstatschat.tumblr.com/post/124987394001/stringsasfactors-sigh)
by Thomas Lumley. If you want to learn more about other approaches to
working with factors and categorical data, I recommend [*Wrangling
categorical data in R*](https://peerj.com/preprints/3163/), by Amelia
McNamara and Nicholas Horton.

## Getting help

If you encounter a clear bug, please file a minimal reproducible example
on [Github](https://github.com/tidyverse/forcats/issues). For questions
and other discussion, please use <https://forum.posit.co/>.

#' Anonymise factor levels
#'
#' Replaces factor levels with arbitrary numeric identifiers. Neither
#' the values nor the order of the levels are preserved.
#'
#' @param f A factor.
#' @param prefix A character prefix to insert in front of the random labels.
#' @export
#' @examples
#' gss_cat$relig %>% fct_count()
#' gss_cat$relig %>% fct_anon() %>% fct_count()
#' gss_cat$relig %>% fct_anon("X") %>% fct_count()
fct_anon <- function(f, prefix = "") {
  levels <- paste0(prefix, zero_pad(seq_len(nlevels(f))))

  f <- lvls_revalue(f, sample(levels))
  lvls_reorder(f, match(levels, levels(f)))
}

digits <- function(x) nchar(max(x, na.rm = TRUE))

zero_pad <- function(x) {
  sprintf(paste0("%0", digits(x), "d"), x)
}

context("test-anon.R")

test_that("new levels are padded numerics", {
  f1 <- factor(letters[1:10])
  f2 <- fct_anon(f1)

  expect_equal(levels(f2), sprintf("%02d", 1:10))
})

test_that("prefix added to start of level", {
  f1 <- factor("x")
  f2 <- fct_anon(f1, prefix = "X")

  expect_equal(levels(f2), "X1")
})

#' Combine levels from two or more factors to create a new factor
#'
#' Computes a factor whose levels are all the combinations of the levels of the input factors.
#'
#' @param ...  <[`dynamic-dots`][rlang::dyn-dots]> Additional factors
#'   or character vectors.
#' @param sep A character string to separate the levels
#' @param keep_empty If TRUE, keep combinations with no observations as levels
#' @return The new factor
#'
#' @export
#' @examples
#' fruit <- factor(c("apple", "kiwi", "apple", "apple"))
#' colour <- factor(c("green", "green", "red", "green"))
#' eaten <- c("yes", "no", "yes", "no")
#' fct_cross(fruit, colour)
#' fct_cross(fruit, colour, eaten)
#' fct_cross(fruit, colour, keep_empty = TRUE)
fct_cross <- function(..., sep = ":", keep_empty = FALSE) {

  flist <- list2(...)
  if (length(flist) == 0) {
    return(factor())
  }

  .data <- tibble::as_tibble(flist, .name_repair = "minimal")
  .data <- lapply(.data, check_factor)

  newf <- exec(paste, !!!.data, sep = sep)

  old_levels <- lapply(.data, levels)
  grid <- exec(expand.grid, old_levels)
  new_levels <- exec(paste, !!!grid, sep = sep)

  if (!keep_empty) {
    new_levels <- intersect(new_levels, newf)
  }
  factor(newf, levels = new_levels)
}

context("fct_cross")

test_that("empty input returns empty factor", {
  expect_equal(fct_cross(), factor())
})

test_that("gives correct levels", {
  fruit <- as_factor(c("apple", "kiwi", "apple", "apple"))
  colour <- as_factor(c("green", "green", "red", "green"))
  f2 <- fct_cross(fruit, colour)

  expect_setequal(levels(f2), c("apple:green", "kiwi:green", "apple:red"))
})

test_that("recycle inputs", {
  expect_length(fct_cross("a", c("a", "b", "c"), "d"), 3)
  expect_error(fct_cross(c("a", "b", "c"), c("a", "b")), "recycle", class = "error")
})

test_that("keeps empty levels when requested", {
  fruit <- as_factor(c("apple", "kiwi", "apple", "apple"))
  colour <- as_factor(c("green", "green", "red", "green"))
  f2 <- fct_cross(fruit, colour, keep_empty = TRUE)

  expect_setequal(levels(f2), c("apple:green", "kiwi:green", "apple:red", "kiwi:red"))
})

test_that("order of levels is preserved", {
  fruit <- as_factor(c("apple", "kiwi", "apple", "apple"))
  colour <- as_factor(c("green", "green", "red", "green"))

  fruit <- fct_relevel(fruit, c("kiwi", "apple"))
  colour <- fct_relevel(colour, c("red", "green"))

  f2 <- fct_cross(fruit, colour)

  expect_setequal(levels(f2), c("kiwi:green", "apple:red", "apple:green"))
})

test_that("gives NA output on NA input", {
  fruit <- as_factor(c("apple", "kiwi", "apple", "apple"))
  colour <- as_factor(c("green", "green", "red", "green"))
  fruit[1] <- NA
  f2 <- fct_cross(fruit, colour)

  expect_true(is.na(f2[1]))
})


test_that("gives NA output on NA input, when keeping empty levels", {
  fruit <- as_factor(c("apple", "kiwi", "apple", "apple"))
  colour <- as_factor(c("green", "green", "red", "green"))
  fruit[1] <- NA
  f2 <- fct_cross(fruit, colour, keep_empty = TRUE)

  expect_true(is.na(f2[1]))
})

test_that("can combine more than two factors", {
  fruit <- as_factor(c("apple", "kiwi", "apple", "apple"))
  colour <- as_factor(c("green", "green", "red", "green"))
  eaten <- c("yes", "no", "yes", "no")

  f2 <- fct_cross(fruit, colour, eaten)

  expect_setequal(levels(f2), c("apple:green:no", "apple:green:yes", "apple:red:yes", "kiwi:green:no"))
})

#' Test for presence of levels in a factor
#'
#' Do any of `lvls` occur in `f`? Compared to [%in%], this function validates
#' `lvls` to ensure that they're actually present in `f`. In other words,
#' `x %in% "not present"` will return `FALSE`, but `fct_match(x, "not present")`
#' will throw an error.
#'
#' @rdname fct_match
#' @param f A factor (or character vector).
#' @param lvls A character vector specifying levels to look for.
#' @return A logical vector
#' @export
#' @examples
#' table(fct_match(gss_cat$marital, c("Married", "Divorced")))
#'
#' # Compare to %in%, misspelled levels throw an error
#' table(gss_cat$marital %in% c("Maried", "Davorced"))
#' \dontrun{
#' table(fct_match(gss_cat$marital, c("Maried", "Davorced")))
#' }
#'
fct_match <- function(f, lvls) {
  f <- check_factor(f)

  bad_lvls <- setdiff(lvls, levels(f))
  bad_lvls <- bad_lvls[!is.na(bad_lvls)]
  if (length(bad_lvls) > 0) {
    stop(
      "Levels not present in factor: ",
      paste0(encodeString(bad_lvls, quote = '"'), collapse = ", "),
      call. = FALSE
    )
  }

  f %in% lvls
}

context("test-fct_match.R")

test_that("equivalent to %in% when levels present", {
  f <- factor(c("a", "b", "c", NA))
  expect_equal(fct_match(f, "a"), f %in% "a")
  expect_equal(fct_match(f, NA), f %in% NA)
})

test_that("error when levels are missing", {
  f <- factor(c("a", "b", "c"))
  expect_error(fct_match(f, "d"), "not present")
})

#' Change factor levels by hand
#'
#' @param .f A factor (or character vector).
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]> A sequence of named character
#'   vectors where the name gives the new level, and the value gives the old
#'   level. Levels not otherwise mentioned will be left as is. Levels can
#'   be removed by naming them `NULL`.
#' @export
#' @examples
#' x <- factor(c("apple", "bear", "banana", "dear"))
#' fct_recode(x, fruit = "apple", fruit = "banana")
#'
#' # If you make a mistake you'll get a warning
#' fct_recode(x, fruit = "apple", fruit = "bananana")
#'
#' # If you name the level NULL it will be removed
#' fct_recode(x, NULL = "apple", fruit = "banana")
#'
#' # Wrap the left hand side in quotes if it contains special variables
#' fct_recode(x, "an apple" = "apple", "a bear" = "bear")
#'
#' # When passing a named vector to rename levels use !!! to splice
#' x <- factor(c("apple", "bear", "banana", "dear"))
#' levels <- c(fruit = "apple", fruit = "banana")
#' fct_recode(x, !!!levels)
fct_recode <- function(.f, ...) {
  f <- check_factor(.f)

  new_levels <- check_recode_levels(...)

  # Remove any named NULL and finish if all NULLs
  nulls <- names(new_levels) == "NULL"
  if (any(nulls)) {
    f <- factor(f, levels = setdiff(levels(f), new_levels[nulls]))
    new_levels <- new_levels[!nulls]
  }

  # Match old levels with new levels
  old_levels <- levels(f)
  idx <- match(new_levels, old_levels)

  # Handle levels that don't exist
  if (any(is.na(idx))) {
    bad <- new_levels[is.na(idx)]
    warning("Unknown levels in `f`: ", paste(bad, collapse = ", "), call. = FALSE)

    new_levels <- new_levels[!is.na(idx)]
    idx <- idx[!is.na(idx)]
  }

  old_levels[idx] <- names(new_levels)

  lvls_revalue(f, old_levels)
}

check_recode_levels <- function(...) {
  levels <- rlang::list2(...)

  is_ok <- function(x) is.character(x) && length(x) == 1
  ok <- vapply(levels, is_ok, logical(1))

  if (!all(ok)) {
    stop(
      "Each input to fct_recode must be a single named string. ",
      "Problems at positions: ", paste0(which(!ok), collapse = ", "),
      call. = FALSE
    )
  }

  unlist(levels)
}

context("fct_recode")

test_that("warns about unknown levels", {
  f1 <- factor(c("a", "b"))

  expect_warning(f2 <- fct_recode(f1, d = "e"), "Unknown levels")
  expect_equal(levels(f2), levels(f1))
})

test_that("can collapse levels", {
  f1 <- factor(c("a1", "a2", "b1", "b2"))
  f2 <- factor(c("a", "a", "b", "b"))

  expect_equal(fct_recode(f1, a = "a1", a = "a2", b = "b1", b = "b2"), f2)
})

test_that("can recode multiple levels to NA", {
  f1 <- factor(c("a1", "empty", "a2", "b", "missing"))
  f2 <- factor(c("a", NA, "a", "b", NA))

  expect_equal(fct_recode(f1, NULL = "missing", NULL = "empty", a = "a1", a = "a2"), f2)
})

test_that("can just remove levels", {
  f1 <- factor(c("a", "missing"))
  f2 <- factor(c("a", NA))

  expect_equal(fct_recode(f1, NULL = "missing"), f2)
})


# check_recode_levels -----------------------------------------------------

test_that("new levels must be character", {
  expect_error(check_recode_levels(a = 1), "Problems at positions: 1")
})

test_that("new levels must be length 1", {
  expect_error(check_recode_levels(a = c("a", "b")), "Problems at positions: 1")
})

#' Convert input to a factor
#'
#' Compared to base R, when `x` is a character, this function creates
#' levels in the order in which they appear, which will be the same on every
#' platform. (Base R sorts in the current locale which can vary from place
#' to place.) When `x` is numeric, the ordering is based on the numeric
#' value and consistent with base R.
#'
#' This is a generic function.
#'
#' @param x Object to coerce to a factor.
#' @param ... Other arguments passed down to method.
#' @export
#' @examples
#' # Character object
#' x <- c("a", "z", "g")
#' as_factor(x)
#' as.factor(x)
#'
#' # Character object containing numbers
#' y <- c("1.1", "11", "2.2", "22")
#' as_factor(y)
#' as.factor(y)
#'
#' # Numeric object
#' z <- as.numeric(y)
#' as_factor(z)
#' as.factor(z)
as_factor <- function(x, ...) {
  ellipsis::check_dots_used()
  UseMethod("as_factor")
}

#' @rdname as_factor
#' @export
as_factor.factor <- function(x, ...) {
  x
}

#' @rdname as_factor
#' @export
as_factor.character <- function(x, ...) {
  # Preserve label for future haven compatibility
  structure(
    fct_inorder(x),
    label = attr(x, "label", exact = TRUE)
  )
}

#' @rdname as_factor
#' @export
as_factor.numeric <- function(x, ...) {
  factor(x)
}

#' @rdname as_factor
#' @export
as_factor.logical <- function(x, ...) {
  factor(x, levels = c("FALSE", "TRUE"))
}

context("as_factor")

test_that("equivalent to fct_inorder", {
  x <- c("a", "z", "g")
  expect_equal(as_factor(x), fct_inorder(x))
})

test_that("leaves factors as is", {
  f1 <- factor(letters)
  f2 <- as_factor(f1)

  expect_identical(f1, f2)
})

test_that("logical has fixed levels", {
  f <- as_factor(FALSE)
  expect_equal(levels(f), c("FALSE", "TRUE"))
})

test_that("supports NA (#89)", {
  x <- c("a", "z", "g", NA)
  expect_equal(as_factor(x), fct_inorder(x))
})

#' Unique values of a factor
#'
#' @param f A factor.
#' @export
#' @examples
#' f <- factor(letters[rpois(100, 10)])
#'
#' unique(f)     # in order of appearance
#' fct_unique(f) # in order of levels
fct_unique <- function(f) {
  factor(levels(f), levels(f), exclude = NULL, ordered = is.ordered(f))
}

#' Pipe operator
#'
#' See \code{\link[magrittr]{\%>\%}} for more details.
#'
#' @name %>%
#' @rdname pipe
#' @keywords internal
#' @export
#' @importFrom magrittr %>%
#' @usage lhs \%>\% rhs
NULL


check_factor <- function(f) {
  if (is.character(f)) {
    factor(f)
  } else if (is.factor(f)) {
    f
  } else {
    stop("`f` must be a factor (or character vector).", call. = FALSE)
  }
}

check_factor_list <- function(fs, arg_name = "fs") {
  if (!is.list(fs)) {
    stop("`fs` must be a list", call. = FALSE)
  }

  is_factor <- vapply(fs, is.factor, logical(1))
  if (any(!is_factor)) {
    stop("All elements of `", arg_name, "` must be factors", call. = FALSE)
  }

  fs
}

context("test-utils.R")

test_that("check_factor() fails when needed", {
  expect_error(check_factor(NA), "factor")
})

test_that("check_factor_list() fails when needed", {
  expect_error(check_factor_list(1), "list")
  expect_error(check_factor_list(list(1)), "factor")
})

#' Make missing values explicit
#'
#' This gives missing values an explicit factor level, ensuring that they
#' appear in summaries and on plots.
#'
#' @param f A factor (or character vector).
#' @param na_level Level to use for missing values: this is what NAs will be changed to.
#' @export
#' @examples
#' f1 <- factor(c("a", "a", NA, NA, "a", "b", NA, "c", "a", "c", "b"))
#' fct_count(f1)
#'
#' f2 <- fct_explicit_na(f1)
#' fct_count(f2)
fct_explicit_na <- function(f, na_level = "(Missing)") {
  f <- check_factor(f)

  is_missing <- is.na(f)
  is_missing_level <- is.na(levels(f))

  if (any(is_missing)) {
    f <- fct_expand(f, na_level)
    f[is_missing] <- na_level

    f
  } else if (any(is_missing_level)) {
    levs <- levels(f)
    levs[is.na(levs)] <- na_level

    lvls_revalue(f, levs)
  } else {
    f
  }
}

context("test-explicit_na.R")

test_that("factor unchanged if no missing levels", {
  f1 <- factor(letters[1:3])
  f2 <- fct_explicit_na(f1)

  expect_identical(f1, f2)
})

test_that("converts implicit NA", {
  f1 <- factor(c("a", NA))
  f2 <- fct_explicit_na(f1)

  expect_equal(f2, fct_inorder(c("a", "(Missing)")))
})

test_that("converts explicit NA", {
  f1 <- factor(c("a", NA), exclude = NULL)
  f2 <- fct_explicit_na(f1)

  expect_equal(f2, fct_inorder(c("a", "(Missing)")))
})

#' Lump together factor levels into "other"
#'
#' @description
#' A family for lumping together levels that meet some criteria.
#' * `fct_lump_min()`: lumps levels that appear fewer than `min` times.
#' * `fct_lump_prop()`: lumps levels that appear in fewer `prop * n` times.
#' * `fct_lump_n()` lumps all levels except for the `n` most frequent
#'    (or least frequent if `n < 0`)
#' * `fct_lump_lowfreq()` lumps together the least frequent levels, ensuring
#'    that "other" is still the smallest level.
#'
#' `fct_lump()` exists primarily for historical reasons, as it automatically
#' picks between these different methods depending on its arguments.
#' We no longer recommend that you use it.
#'
#' @param f A factor (or character vector).
#' @param n Positive `n` preserves the most common `n` values.
#'   Negative `n` preserves the least common `-n` values.
#'   It there are ties, you will get at least `abs(n)` values.
#' @param prop  Positive `prop` lumps values which do not appear at least
#'   `prop` of the time. Negative `prop` lumps values that
#'   do not appear at most `-prop` of the time.
#' @param min Preserve levels that appear at least `min` number of times.
#' @param w An optional numeric vector giving weights for frequency of
#'   each value (not level) in f.
#' @param other_level Value of level used for "other" values. Always
#'   placed at end of levels.
#' @param ties.method A character string specifying how ties are
#'   treated. See [rank()] for details.
#' @export
#' @seealso [fct_other()] to convert specified levels to other.
#' @examples
#' x <- factor(rep(LETTERS[1:9], times = c(40, 10, 5, 27, 1, 1, 1, 1, 1)))
#' x %>% table()
#' x %>% fct_lump_n(3) %>% table()
#' x %>% fct_lump_prop(0.10) %>% table()
#' x %>% fct_lump_min(5) %>% table()
#' x %>% fct_lump_lowfreq() %>% table()
#'
#' x <- factor(letters[rpois(100, 5)])
#' x
#' table(x)
#' table(fct_lump_lowfreq(x))
#'
#' # Use positive values to collapse the rarest
#' fct_lump_n(x, n = 3)
#' fct_lump_prop(x, prop = 0.1)
#'
#' # Use negative values to collapse the most common
#' fct_lump_n(x, n = -3)
#' fct_lump_prop(x, prop = -0.1)
#'
#' # Use weighted frequencies
#' w <- c(rep(2, 50), rep(1, 50))
#' fct_lump_n(x, n = 5, w = w)
#'
#' # Use ties.method to control how tied factors are collapsed
#' fct_lump_n(x, n = 6)
#' fct_lump_n(x, n = 6, ties.method = "max")
#'
#' # Use fct_lump_min() to lump together all levels with fewer than `n` values
#' table(fct_lump_min(x, min = 10))
#' table(fct_lump_min(x, min = 15))
fct_lump <- function(f, n, prop, w = NULL, other_level = "Other",
                     ties.method = c("min", "average", "first", "last", "random", "max")) {

  ties.method <- match.arg(ties.method)
  check_calc_levels(f, w)


  if (missing(n) && missing(prop)) {
    fct_lump_lowfreq(f, other_level = other_level)
  } else if (missing(prop)) {
    fct_lump_n(f, n, w, other_level, ties.method)
  } else if (missing(n)) {
    fct_lump_prop(f, prop, w, other_level)
  } else {
    abort("Must supply only one of `n` and `prop`")
  }
}

#' @export
#' @rdname fct_lump
fct_lump_min <- function(f, min, w = NULL, other_level = "Other") {

  calcs <- check_calc_levels(f, w)
  f <- calcs$f

  if (!is.numeric(min) || length(min) != 1 || min < 0) {
    rlang::abort("`min` must be a positive number")
  }

  new_levels <- ifelse(calcs$count >= min, levels(f), other_level)

  if (other_level %in% new_levels) {
    f <- lvls_revalue(f, new_levels)
    fct_relevel(f, other_level, after = Inf)
  } else {
    f
  }

}

#' @export
#' @rdname fct_lump
fct_lump_prop <- function(f, prop, w = NULL, other_level = "Other") {

  calcs <- check_calc_levels(f, w)
  f <- calcs$f

  if (!is.numeric(prop) || length(prop) != 1) {
    rlang::abort("`prop` must be a number")
  }

  prop_n <- calcs$count / calcs$total
  if (prop < 0) {
    new_levels <- ifelse(prop_n <= -prop, levels(f), other_level)
  } else {
    new_levels <- ifelse(prop_n > prop, levels(f), other_level)
  }

  if (prop > 0 && sum(prop_n <= prop) <= 1) {
    # No lumping needed
    return(f)
  }

  if (other_level %in% new_levels) {
    f <- lvls_revalue(f, new_levels)
    fct_relevel(f, other_level, after = Inf)
  } else {
    f
  }
}


#' @export
#' @rdname fct_lump
fct_lump_n <- function(f, n, w = NULL, other_level = "Other",
                       ties.method = c("min", "average", "first", "last", "random", "max")) {

  ties.method <- match.arg(ties.method)
  calcs <- check_calc_levels(f, w)
  f <- calcs$f

  if (!is.numeric(n) || length(n) != 1) {
    rlang::abort("`n` must be a number")
  }

  if (n < 0) {
    rank <- rank(calcs$count, ties.method = ties.method)
    n <- -n
  } else {
    rank <- rank(-calcs$count, ties.method = ties.method)
  }

  new_levels <- ifelse(rank <= n, levels(f), other_level)

  if (sum(rank > n) <= 1) {
    # No lumping needed
    return(f)
  }

  if (other_level %in% new_levels) {
    f <- lvls_revalue(f, new_levels)
    fct_relevel(f, other_level, after = Inf)
  } else {
    f
  }

}

#' @export
#' @rdname fct_lump
fct_lump_lowfreq <- function(f, other_level = "Other") {

  calcs <- check_calc_levels(f, NULL)
  f <- calcs$f

  new_levels <- ifelse(!in_smallest(calcs$count), levels(f), other_level)

  if (other_level %in% new_levels) {
    f <- lvls_revalue(f, new_levels)
    fct_relevel(f, other_level, after = Inf)
  } else {
    f
  }

}

check_calc_levels <- function(f, w = NULL) {
  f <- check_factor(f)
  w <- check_weights(w, length(f))

  if (is.null(w)) {
    count <- as.vector(table(f))
    total <- length(f)
  } else {
    count <- as.vector(tapply(w, f, FUN = sum))
    total <- sum(w)
  }
  list(f = f, count = count, total = total)
}

# Lump together smallest groups, ensuring that the collective
# "other" is still the smallest group. Assumes x is vector
# of counts in descending order
lump_cutoff <- function(x) {
  left <- sum(x)

  for (i in seq_along(x)) {
    # After group, there are this many left
    left <- left - x[i]

    if (x[i] > left)
      return(i + 1)
  }

  length(x) + 1
}

# Given vector of counts, returns logical vector if in
# smallest groups
in_smallest <- function(x) {
  ord_x <- order(x, decreasing = TRUE)
  idx <- lump_cutoff(x[ord_x])

  to_lump <- seq_along(x) >= idx
  # Undo initial ordering
  to_lump[order(ord_x)]
}

check_weights <- function(w, n = length(w)) {
  if (is.null(w)) {
    return(w)
  }

  if (!is.numeric(w)) {
    stop("`w` must be a numeric vector", call. = FALSE)
  }

  if (length(w) != n) {
    stop(
      "`w` must be the same length as `f` (", n, "), not length ", length(w),
      call. = FALSE
    )
  }

  bad <- w < 0 | is.na(w)
  if (any(bad)) {
    stop(
      "All `w` must be non-negative and non-missing. Problems at positions: ",
      paste0(which(bad), collapse = ", "),
      call. = FALSE
    )
  }

  w
}

context("fct_lump")

test_that("too many arguments fails", {
  f <- c("a", "b", "c")
  expect_error(fct_lump(f, n = 1, count = 1))
  expect_error(fct_lump(f, n = 1, prop = 0.1))
  expect_error(fct_lump(f, min = 2, count = 1))
})

test_that("positive values keeps most commmon", {
  f <- c("a", "a", "a", "b", "b", "c", "d", "e", "f", "g")

  expect_equal(levels(fct_lump(f, n = 1)), c("a", "Other"))
  expect_equal(levels(fct_lump(f, n = 2)), c("a", "b", "Other"))

  expect_equal(levels(fct_lump(f, prop = 0.25)), c("a", "Other"))
  expect_equal(levels(fct_lump(f, prop = 0.15)), c("a", "b", "Other"))
})

test_that("ties are respected", {
  f <- c("a", "a", "a", "b", "b", "b", "c", "d")
  expect_equal(levels(fct_lump(f, 1)), c("a", "b", "Other"))
})

test_that("negative values drop most common" ,{
  f <- c("a", "a", "a", "a", "b", "b", "b", "b", "c", "d")
  expect_equal(levels(fct_lump(f, n = -1)), c("c", "d", "Other"))
  expect_equal(levels(fct_lump(f, prop = -0.2)), c("c", "d", "Other"))
})

test_that("return original factor when all element satisfy n / p condition", {
  f <- c("a", "a", "a", "b", "b", "c", "d", "e", "f", "g")

  expect_equal(levels(fct_lump(f, n = 4)), c("a", "b", "c", "d", "e", "f", "g"))
  expect_equal(levels(fct_lump(f, n = 10)), c("a", "b", "c", "d", "e", "f", "g"))
  expect_equal(levels(fct_lump(f, n = -10)), c("a", "b", "c", "d", "e", "f", "g"))

  expect_equal(levels(fct_lump(f, prop = 0.01)), c("a", "b", "c", "d", "e", "f", "g"))
  expect_equal(levels(fct_lump(f, prop = -1)), c("a", "b", "c", "d", "e", "f", "g"))
})

test_that("different behaviour when apply tie function", {
  f <- c("a", "a", "a", "b", "b", "c", "d", "e", "f", "g")

  expect_equal(levels(fct_lump(f, n = 4, ties.method = "min")),
               c("a", "b", "c", "d", "e", "f", "g"))
  expect_equal(levels(fct_lump(f, n = 4, ties.method = "max")),
               c("a", "b", "Other" ))

  # Rank of c, d, e, f, g is (3+4+5+6+7)/5 = 5
  expect_equal(levels(fct_lump(f, n = 4, ties.method = "average")),
               c("a", "b", "Other" ))
  expect_equal(levels(fct_lump(f, n = 5, ties.method = "average")),
               c("a", "b", "c", "d", "e", "f", "g"))

  expect_equal(levels(fct_lump(f, n = 4, ties.method = "first")),
               c("a", "b", "c", "d", "Other"))

  if (getRversion() >= "3.3.0") {
    expect_equal(levels(fct_lump(f, n = 4, ties.method = "last")),
                 c("a", "b", "f", "g", "Other"))
  }
})

test_that("NAs included in total", {
  f <- factor(c("a", "a", "b", "c", rep(NA, 7)))

  o1 <- fct_lump(f, prop = 0.10)
  expect_equal(levels(o1), c("a", "Other"))

  o2 <- fct_lump(f, w = rep(1, 11), prop = 0.10)
  expect_equal(levels(o2), c("a", "Other"))
})

test_that("bad weights generate error messages", {
  expect_error(fct_lump(letters, w = letters), "must be a numeric vector")
  expect_error(fct_lump(letters, w = 1:10), "must be the same length")
  expect_error(fct_lump(letters, w = rep(-1, 26)), "must be non-negative")
})

test_that("values are correctly weighted", {
  f <- c("a", "a", "a", "b", "b", "c", "d", "e", "f", "g")
  w <- c( 0.2, 0.2, 0.6, 2,   2,   6,   4,   2,   2,   1)
  f2 <- c(
    "a",
    rep("b", 4),
    rep("c", 6),
    rep("d", 4),
    rep("e", 2),
    rep("f", 2),
    "g"
  )

  expect_equal(levels(fct_lump(f, w = w)), levels(fct_lump(f2)))
  expect_equal(
    levels(fct_lump(f, n = 1, w = w)),
    levels(fct_lump(f2, n = 1))
  )
  expect_equal(
    levels(fct_lump(f, n = -2, w = w, ties.method = "first")),
    levels(fct_lump(f2, n = -2, ties.method = "first"))
  )
  expect_equal(
    levels(fct_lump(f, n = 99, w = w)),
    levels(fct_lump(f2, n = 99))
  )
  expect_equal(
    levels(fct_lump(f, prop = 0.01, w = w)),
    levels(fct_lump(f2, prop = 0.01))
  )
  expect_equal(
    levels(fct_lump(f, prop = -0.25, w = w, ties.method = "max")),
    levels(fct_lump(f2, prop = -0.25, ties.method = "max"))
  )
})

test_that("do not change the label when no lumping occurs", {
  f <- c("a", "a", "a", "a", "b", "b", "b", "c", "c", "d")
  expect_equal(levels(fct_lump(f, n = 3)), c("a", "b", "c", "d"))
  expect_equal(levels(fct_lump(f, prop = 0.1)), c("a", "b", "c", "d"))
})

test_that("only have one small other level", {
  f <- c("a", "a", "a", "a", "b", "b", "b", "c", "c", "d")
  expect_equal(levels(fct_lump(f)), c("a", "b", "c", "Other"))
})

test_that("fct_lump_min works when not weighted", {
  f <- c("a", "a", "a", "b", "b", "c", "d", "e", "f", "g")

  expect_equal(levels(fct_lump_min(f, min = 3)), c("a", "Other"))
  expect_equal(levels(fct_lump_min(f, min = 2)), c("a", "b", "Other"))
})

test_that("fct_lump_min works when weighted", {
  f <- c("a", "b", "c", "d", "e")
  w <- c( 0.2, 2,   6,   4,   1)

  expect_equal(levels(fct_lump_min(f, min = 6, w = w)), c("c", "Other"))
  expect_equal(levels(fct_lump_min(f, min = 1.5, w = w)), c("b", "c", "d", "Other"))
})

test_that("throws error if n or prop is not numeric", {
  f <- c("a", "a", "a", "a", "b", "b", "b", "c", "c", "d")

  expect_error(fct_lump(f, n = "2"), "`n`")
  expect_error(fct_lump(f, prop = "2"), "`prop`")
})

test_that("fct_lump_prop works when not weighted", {
  f <- c("a", "a", "a", "b", "b", "c", "d", "e", "f", "g")

  expect_equal(levels(fct_lump_prop(f, prop = 0.2)), c("a", "Other"))
  expect_equal(levels(fct_lump_prop(f, prop = 0.1)), c("a", "b", "Other"))
})

test_that("fct_lump_prop works when weighted", {
  f <- c("a", "b", "c", "d", "e")
  w <- c( 0.2, 2,   6,   4,   1)

  expect_equal(levels(fct_lump_prop(f, prop = 0.3, w = w)), c("c", "d", "Other"))
  expect_equal(levels(fct_lump_prop(f, prop = 0.2, w = w)), c("c", "d", "Other"))
})

# Default -----------------------------------------------------------------

test_that("lumps smallest", {
  expect_equal(lump_test(c(1, 2, 3, 6)), "Xbcd")
  expect_equal(lump_test(c(1, 2, 3, 7)), "XXXd")

  expect_equal(lump_test(c(1, 2, 3, 7, 13)), "XXXde")
  expect_equal(lump_test(c(1, 2, 3, 7, 14)), "XXXXe")
})

test_that("doesn't lump if none small enough", {
  expect_equal(lump_test(c(2, 2, 4)), "abc")
})

test_that("order doesn't matter", {
  expect_equal(lump_test(c(2, 2, 5)), "XXc")
  expect_equal(lump_test(c(2, 5, 2)), "XbX")
  expect_equal(lump_test(c(5, 2, 2)), "aXX")
})

#' Randomly permute factor levels
#'
#' @param f A factor (or character vector).
#' @export
#' @examples
#' f <- factor(c("a", "b", "c"))
#' fct_shuffle(f)
#' fct_shuffle(f)
fct_shuffle <- function(f) {
  f <- check_factor(f)

  lvls_reorder(f, sample(lvls_seq(f)))
}

#' Collapse factor levels into manually defined groups
#'
#' @param .f A factor (or character vector).
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]> A series of named character vectors. The levels in
#'   each vector will be replaced with the name.
#' @inheritParams fct_other
#' @param group_other Deprecated. Replace all levels not named in `...` with "Other"?
#' @export
#' @examples
#' fct_count(gss_cat$partyid)
#'
#' partyid2 <- fct_collapse(gss_cat$partyid,
#'   missing = c("No answer", "Don't know"),
#'   other = "Other party",
#'   rep = c("Strong republican", "Not str republican"),
#'   ind = c("Ind,near rep", "Independent", "Ind,near dem"),
#'   dem = c("Not str democrat", "Strong democrat")
#' )
#' fct_count(partyid2)
fct_collapse <- function(.f, ..., other_level = NULL, group_other = "DEPRECATED") {

  if (!missing(group_other)) {
    warning("`group_other` is deprecated. Please use `other_level` instead")
    if (isTRUE(group_other) && is.null(other_level)) {
      other_level <- "Other"
    }
  }

  new <- rlang::list2(...)
  levs <- as.list(unlist(new, use.names = FALSE))

  if (!is.null(other_level)){
    f <- check_factor(.f)
    levels <- levels(f)
    new[[other_level]] <- levels[!levels %in% levs]
    levs <- c(levs, new[[other_level]])
  }

  names(levs) <- names(new)[rep(seq_along(new), vapply(new, length, integer(1)))]
  out <- fct_recode(.f, !!!levs)

  if (any(levels(out) == other_level)){
    fct_relevel(out, other_level, after = Inf)
  } else return(out)

}

context("test-collapse.R")

test_that("can collapse multiple values", {
  f1 <- factor(letters[1:3])
  f2 <- fct_collapse(f1, x = c("a", "b"), y = "c")

  expect_equal(f2, factor(c("x", "x", "y")))
})

test_that("empty dots yields unchanged factor", {
  f1 <- factor(letters[1:3])
  f2 <- fct_collapse(f1)

  expect_identical(f1, f2)
})

test_that("can collapse missing levels", {
  f1 <- factor(c("x", NA), exclude = NULL)
  f2 <- fct_collapse(f1, y = NA_character_)

  expect_equal(f2, factor(c("x", "y")))
})

test_that("can collapse un-named levels to Other", {
  f1 <- factor(letters[1:3])
  f2 <- fct_collapse(f1, xy = c("a", "b"), other_level = "Other")

  expect_equal(f2, factor(c("xy", "xy", "Other"), levels = c("xy", "Other")))
})

test_that("collapses levels correctly when group_other is TRUE but no other variables to group", {
  f1 <- factor(letters[1:4])
  f2 <- fct_collapse(f1, x1 = c("a", "b", "d"), x2 = "c", other_level = "Other")

  expect_equal(f2, factor(c("x1", "x1", "x2", "x1"), levels = c("x1", "x2")))
})

test_that("collapses levels correctly when group_other is TRUE and some Other variables to group", {
  f1 <- factor(letters[1:4])
  f2 <- fct_collapse(f1, x1 = c("a", "d"), x2 = "c", other_level = "Other")

  expect_equal(f2, factor(c("x1", "Other", "x2", "x1"), levels = c("x1", "x2", "Other")))
})

test_that("does not automatically collapse unnamed levels to Other", {
  f1 <- factor(letters[1:3])
  f2 <- fct_collapse(f1, xy = c("a", "b"))

  expect_equal(f2, factor(c("xy", "xy", "c"), levels = c("xy", "c")))
})

test_that("group_other is deprecated", {
  f1 <- factor(letters[1:4])
  f2 <- expect_warning(
    fct_collapse(f1, x1 = c("a", "d"), x2 = "c", group_other = TRUE),
    "deprecated"
  )

  expect_equal(levels(f2), c("x1", "x2", "Other"))
})

#' Count entries in a factor
#'
#' @param f A factor (or character vector).
#' @param sort If `TRUE`, sort the result so that the most common values
#'   float to the top.
#' @param prop If `TRUE`, compute the fraction of marginal table.
#' @return A tibble with columns `f`, `n` and `p`, if prop is `TRUE`.
#' @export
#' @examples
#' f <- factor(sample(letters)[rpois(1000, 10)])
#' table(f)
#' fct_count(f)
#' fct_count(f, sort = TRUE)
#' fct_count(f, sort = TRUE, prop = TRUE)
fct_count <- function(f, sort = FALSE, prop = FALSE) {
  f2 <- check_factor(f)
  n_na <- sum(is.na(f))

  df <- tibble::tibble(
    f = fct_inorder(c(levels(f2), if (n_na > 0) NA)),
    n = c(tabulate(f2, nlevels(f)), if (n_na > 0) n_na)
  )

  if (sort) {
    df <- df[order(df$n, decreasing = TRUE), ]
  }

  if (prop) {
    df$p <- prop.table(df$n)
  }

  df
}

context("test-count.R")

test_that("0 count for empty levels", {
  f <- factor(levels = c("a", "b"))
  expect_equal(fct_count(f)$n, c(0, 0))

  f <- factor("a", levels = c("a", "b", "c"))
  expect_equal(fct_count(f)$n, c(1, 0, 0))
})

test_that("counts NA even when not in levels", {
  f <- factor(c("a", "a", NA))
  out <- fct_count(f)

  expect_equal(out$n, c(2, 1))
  # and doesn't change levels
  expect_equal(levels(out$f), levels(f))
})

test_that("returns marginal table", {
  f <- factor(c("a", "a", "b"))
  out <- fct_count(f, prop = TRUE)

  expect_equal(out$n, c(2, 1))
  expect_equal(out$p, c(2, 1) / 3)
})

test_that("sort = TRUE brings most frequent values to top", {
  f <- factor(c("a", "b", "b"))
  out <- fct_count(f, sort = TRUE)

  expect_equal(out$f, factor(c("b", "a"), levels = c("a", "b")))
})

#' Shift factor levels to left or right, wrapping around at end
#'
#' This is useful when the levels of an ordered factor are actually cyclical,
#' with different conventions on the starting point.
#'
#' @param f A factor.
#' @param n Positive values shift to the left; negative values shift to
#'   the right.
#' @export
#' @examples
#' x <- factor(
#'   c("Mon", "Tue", "Wed"),
#'   levels = c("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"),
#'   ordered = TRUE
#' )
#' x
#' fct_shift(x)
#' fct_shift(x, 2)
#' fct_shift(x, -1)
fct_shift <- function(f, n = 1L) {
  lvls_reorder(f, shift(nlevels(f), n))
}

shift <- function(m, n) {
  stopifnot(is.numeric(m), length(m) == 1L)
  stopifnot(is.numeric(n), length(n) == 1L)

  ((seq_len(m) - 1) + n) %% m + 1
}

context("test-shift.R")

test_that("can shift in either direction", {
  f1 <- factor(c("a", "b", "c"))

  f2_l <- fct_shift(f1, 1)
  expect_equal(levels(f2_l), c("b", "c", "a"))

  f2_r <- fct_shift(f1, -1)
  expect_equal(levels(f2_r), c("c", "a", "b"))
})

test_that("0 shift leaves unchanged", {
  f1 <- factor(c("a", "b", "c"))
  f2 <- fct_shift(f1, 0)

  expect_identical(f1, f2)
})

#' Replace levels with "other"
#'
#' @inheritParams fct_lump
#' @param keep,drop Pick one of `keep` and `drop`:
#'   * `keep` will preserve listed levels, replacing all others with
#'     `other_level`.
#'   * `drop` will replace listed levels with `other_level`, keeping all
#'     as is.
#' @seealso [fct_lump()] to automatically convert the rarest (or most
#'    common) levels to "other".
#' @export
#' @examples
#' x <- factor(rep(LETTERS[1:9], times = c(40, 10, 5, 27, 1, 1, 1, 1, 1)))
#'
#' fct_other(x, keep = c("A", "B"))
#' fct_other(x, drop = c("A", "B"))
fct_other <- function(f, keep, drop, other_level = "Other") {
  f <- check_factor(f)

  if (!xor(missing(keep), missing(drop))) {
    stop("Must supply exactly one of `keep` and `drop`", call. = FALSE)
  }

  levels <- levels(f)
  if (!missing(keep)) {
    levels[!levels %in% keep] <- other_level
  } else {
    levels[levels %in% drop] <- other_level
  }

  f <- lvls_revalue(f, levels)
  fct_relevel(f, other_level, after = Inf)
}

context("fct_other")

test_that("keeps levels in keep", {
  x1 <- factor(c("a", "b"))
  x2 <- fct_other(x1, keep = "a")

  expect_equal(levels(x2), c("a", "Other"))
})

test_that("drops levels in drop", {
  x1 <- factor(c("a", "b"))
  x2 <- fct_other(x1, drop = "a")

  # other always placed at end
  expect_equal(levels(x2), c("b", "Other"))
})

test_that("must supply exactly one of drop and keep", {
  f <- factor(c("a", "b"))

  expect_error(fct_other(f), "exactly one")
  expect_error(fct_other(f, keep = "a", drop = "a"), "exactly one")
})

#' Reorder factor levels by sorting along another variable
#'
#' `fct_reorder()` is useful for 1d displays where the factor is mapped to
#' position; `fct_reorder2()` for 2d displays where the factor is mapped to
#' a non-position aesthetic. `last2()` and `first2()` are helpers for `fct_reorder2()`;
#' `last2()` finds the last value of `y` when sorted by `x`; `first2()` finds the first value.
#'
#' @param .f A factor (or character vector).
#' @param .x,.y The levels of `f` are reordered so that the values
#'    of `.fun(.x)` (for `fct_reorder()`) and `fun(.x, .y)` (for `fct_reorder2()`)
#'    are in ascending order.
#' @param .fun n summary function. It should take one vector for
#'   `fct_reorder`, and two vectors for `fct_reorder2`, and return a single
#'   value.
#' @param ... Other arguments passed on to `.fun`. A common argument is
#'   `na.rm = TRUE`.
#' @param .desc Order in descending order? Note the default is different
#'   between `fct_reorder` and `fct_reorder2`, in order to
#'   match the default ordering of factors in the legend.
#' @importFrom stats median
#' @export
#' @examples
#' df <- tibble::tribble(
#'   ~color,     ~a, ~b,
#'   "blue",      1,  2,
#'   "green",     6,  2,
#'   "purple",    3,  3,
#'   "red",       2,  3,
#'   "yellow",    5,  1
#' )
#' df$color <- factor(df$color)
#' fct_reorder(df$color, df$a, min)
#' fct_reorder2(df$color, df$a, df$b)
#'
#' boxplot(Sepal.Width ~ Species, data = iris)
#' boxplot(Sepal.Width ~ fct_reorder(Species, Sepal.Width), data = iris)
#' boxplot(Sepal.Width ~ fct_reorder(Species, Sepal.Width, .desc = TRUE), data = iris)
#'
#' chks <- subset(ChickWeight, as.integer(Chick) < 10)
#' chks <- transform(chks, Chick = fct_shuffle(Chick))
#'
#' if (require("ggplot2")) {
#' ggplot(chks, aes(Time, weight, colour = Chick)) +
#'   geom_point() +
#'   geom_line()
#'
#' # Note that lines match order in legend
#' ggplot(chks, aes(Time, weight, colour = fct_reorder2(Chick, Time, weight))) +
#'   geom_point() +
#'   geom_line() +
#'   labs(colour = "Chick")
#' }
fct_reorder <- function(.f, .x, .fun = median, ..., .desc = FALSE) {
  f <- check_factor(.f)
  stopifnot(length(f) == length(.x))
  ellipsis::check_dots_used()

  summary <- tapply(.x, .f, .fun, ...)
  # This is a bit of a weak test, but should detect the most common case
  # where `.fun` returns multiple values.
  if (is.list(summary)) {
    stop("`fun` must return a single value per group", call. = FALSE)
  }

  lvls_reorder(f, order(summary, decreasing = .desc))
}

#' @export
#' @rdname fct_reorder
fct_reorder2 <- function(.f, .x, .y, .fun = last2, ..., .desc = TRUE) {
  f <- check_factor(.f)
  stopifnot(length(f) == length(.x), length(.x) == length(.y))
  ellipsis::check_dots_used()

  summary <- tapply(seq_along(.x), f, function(i) .fun(.x[i], .y[i], ...))
  if (is.list(summary)) {
    stop("`fun` must return a single value per group", call. = FALSE)
  }

  lvls_reorder(.f, order(summary, decreasing = .desc))
}


#' @export
#' @rdname fct_reorder
last2 <- function(.x, .y) {
  .y[order(.x, na.last = FALSE)][length(.y)]
}

#' @export
#' @rdname fct_reorder
first2 <- function(.x, .y) {
  .y[order(.x)][1]
}



#' Reorder factor levels by first appearance, frequency, or numeric order
#'
#' This family of functions changes only the order of the levels.
#' * `fct_inorder()`: by the order in which they first appear.
#' * `fct_infreq()`: by number of observations with each level (largest first)
#' * `fct_inseq()`: by numeric value of level.
#'
#' @inheritParams lvls_reorder
#' @param f A factor
#' @export
#' @examples
#' f <- factor(c("b", "b", "a", "c", "c", "c"))
#' f
#' fct_inorder(f)
#' fct_infreq(f)
#'
#' f <- factor(1:3, levels = c("3", "2", "1"))
#' f
#' fct_inseq(f)
fct_inorder <- function(f, ordered = NA) {
  f <- check_factor(f)

  idx <- as.integer(f)[!duplicated(f)]
  idx <- idx[!is.na(idx)]
  lvls_reorder(f, idx, ordered = ordered)
}

#' @export
#' @rdname fct_inorder
fct_infreq <- function(f, ordered = NA) {
  f <- check_factor(f)

  lvls_reorder(f, order(table(f), decreasing = TRUE), ordered = ordered)
}

#' @export
#' @rdname fct_inorder
fct_inseq <- function(f, ordered = NA) {
  f <- check_factor(f)

  num_levels <- suppressWarnings(as.numeric(levels(f)))

  if (all(is.na(num_levels))) {
    stop("At least one existing level must be coercible to numeric.", call. = FALSE)
  }

  lvls_reorder(f, order(num_levels), ordered = ordered)
}

context("test-reorder.R")

test_that("can reorder by 2d summary", {
  df <- tibble::tribble(
    ~g,  ~x,
    "a", 3,
    "a", 3,
    "b", 2,
    "b", 2,
    "b", 1
  )

  f1 <- fct_reorder(df$g, df$x)
  expect_equal(levels(f1), c("b", "a"))

  f2 <- fct_reorder(df$g, df$x, .desc = TRUE)
  expect_equal(levels(f2), c("a", "b"))
})

test_that("can reorder by 2d summary", {
  df <- tibble::tribble(
    ~g, ~x, ~y,
    "a", 1, 10,
    "a", 2, 5,
    "b", 1, 5,
    "b", 2, 10
  )

  f1 <- fct_reorder2(df$g, df$x, df$y)
  expect_equal(levels(f1), c("b", "a"))

  f2 <- fct_reorder(df$g, df$x, .desc = TRUE)
  expect_equal(levels(f2), c("a", "b"))
})

test_that("complains if summary doesn't return single value", {
  fun1 <- function(x, y) c(1, 2)
  fun2 <- function(x, y) integer()

  expect_error(fct_reorder("a", 1, fun1), "single value per group")
  expect_error(fct_reorder("a", 1, fun2), "single value per group")
  expect_error(fct_reorder2("a", 1, 2, fun1), "single value per group")
  expect_error(fct_reorder2("a", 1, 2, fun2), "single value per group")
})

test_that("fct_infreq respects missing values", {
  f <- factor(c("a", "b", "b", NA, NA, NA), exclude = FALSE)
  expect_equal(levels(fct_infreq(f)), c(NA, "b", "a"))
})

test_that("fct_inseq sorts in numeric order", {
  x <- c("1", "2", "3")
  expect_equal(
    fct_inseq(factor(x, levels = c("3", "1", "2"))),
    factor(x, levels = c("1", "2", "3"))
  )

  # non-numeric go to end
  x <- c("1", "2", "3", "a")
  expect_equal(
    fct_inseq(factor(x, levels = c("a", "3", "1", "2"))),
    factor(x, levels = c("1", "2", "3", "a"))
  )
})

test_that("fct_inseq gives error for non-numeric levels", {
  f <- factor(c("c", "a", "a", "b"))
  expect_error(levels(fct_inseq(f)), "level must be coercible to numeric")
})

#' Concatenate factors, combining levels
#'
#' This is a useful way of patching together factors from multiple sources
#' that really should have the same levels but don't.
#'
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]> Individual
#'   factors. Uses tidy dots, so you can splice in a list of factors
#'   with `!!!`.
#' @export
#' @examples
#' fa <- factor("a")
#' fb <- factor("b")
#' fab <- factor(c("a", "b"))
#'
#' c(fa, fb, fab)
#' fct_c(fa, fb, fab)
#'
#' # You can also pass a list of factors with !!!
#' fs <- list(fa, fb, fab)
#' fct_c(!!!fs)
fct_c <- function(...) {
  fs <- rlang::list2(...)
  fs <- check_factor_list(fs, "...")

  if (length(fs) == 0) {
    return(factor())
  }

  levels <- lvls_union(fs)
  all <- unlist(fct_unify(fs, levels), use.names = FALSE)
  factor(all, levels = levels, exclude = NULL)
}

#' Unify the levels in a list of factors
#'
#' @param fs A list of factors
#' @param levels Set of levels to apply to every factor. Default to
#'   union of all factor levels
#' @export
#' @examples
#' fs <- list(factor("a"), factor("b"), factor(c("a", "b")))
#' fct_unify(fs)
fct_unify <- function(fs, levels = lvls_union(fs)) {
  fs <- check_factor_list(fs)

  lapply(fs, lvls_expand, new_levels = levels)
}

context("fct_c")

test_that("uses tidy_dots", {
  fs <- list(factor("a"), factor("b"))
  fab <- factor(c("a", "b"))

  expect_equal(fct_c(!!!fs), fab)
  expect_equal(fct_c(fs[[1]], fs[[2]]), fab)
})

test_that("all inputs must be factors", {
  expect_error(fct_c("a"), "must be factors")
})

test_that("empty input yields empty factor", {
  expect_equal(fct_c(), factor())
})

#' Low-level functions for manipulating levels
#'
#' `lvls_reorder` leaves values as they are, but changes the order.
#' `lvls_revalue` changes the values of existing levels; there must
#' be one new level for each old level.
#' `lvls_expand` expands the set of levels; the new levels must
#' include the old levels.
#'
#' These functions are less helpful than the higher-level `fct_` functions,
#' but are safer than the very low-level manipulation of levels directly,
#' because they are more specific, and hence can more carefully check their
#' arguments.
#'
#' @param f A factor (or character vector).
#' @param idx A integer index, with one integer for each existing level.
#' @param new_levels A character vector of new levels.
#' @param ordered A logical which determines the "ordered" status of the
#'   output factor. `NA` preserves the existing status of the factor.
#' @name lvls
#' @examples
#' f <- factor(c("a", "b", "c"))
#' lvls_reorder(f, 3:1)
#' lvls_revalue(f, c("apple", "banana", "carrot"))
#' lvls_expand(f, c("a", "b", "c", "d"))
NULL

#' @export
#' @rdname lvls
lvls_reorder <- function(f, idx, ordered = NA) {
  f <- check_factor(f)
  if (!is.numeric(idx)) {
    stop("`idx` must be numeric", call. = FALSE)
  }
  if (!setequal(idx, lvls_seq(f)) || length(idx) != nlevels(f)) {
    stop("`idx` must contain one integer for each level of `f`", call. = FALSE)
  }

  refactor(f, levels(f)[idx], ordered = ordered)
}

#' @export
#' @rdname lvls
lvls_revalue <- function(f, new_levels) {
  f <- check_factor(f)

  if (!is.character(new_levels)) {
    stop("`new_levels` must be a character vector", call. = FALSE)
  }

  if (length(new_levels) != nlevels(f)) {
    stop(
      "`new_levels` must be the same length as `levels(f)`: expected ",
      nlevels(f), " new levels, got ", length(new_levels), ".",
      call. = FALSE
    )
  }

  if (anyDuplicated(new_levels)) {
    # Collapse levels, creating a new factor
    u_levels <- unique(new_levels)
    index <- match(new_levels, u_levels)

    out <- index[f]
    attributes(out) <- attributes(f)
    attr(out, "levels") <- u_levels
    out
  } else {
    attr(f, "levels") <- new_levels
    f
  }
}

#' @export
#' @rdname lvls
lvls_expand <- function(f, new_levels) {
  f <- check_factor(f)

  missing <- setdiff(levels(f), new_levels)
  if (length(missing) > 0) {
    stop(
      "Must include all existing levels. Missing: ", paste0(missing, collapse = ", "),
      call. = FALSE)
  }

  refactor(f, new_levels)
}

lvls_seq <- function(f) {
  seq_along(levels(f))
}

refactor <- function(f, new_levels, ordered = NA) {
  if (is.na(ordered)) {
    ordered <- is.ordered(f)
  }

  new_f <- factor(f, levels = new_levels, exclude = NULL, ordered = ordered)
  attributes(new_f) <- utils::modifyList(attributes(f), attributes(new_f))
  new_f
}


#' Find all levels in a list of factors
#'
#' @param fs A list of factors.
#' @export
#' @examples
#' fs <- list(factor("a"), factor("b"), factor(c("a", "b")))
#' lvls_union(fs)
lvls_union <- function(fs) {
  fs <- check_factor_list(fs)
  Reduce(function(x, y) union(x, levels(y)), fs, init = character())
}

context("lvls")

# lvls_expand -------------------------------------------------------------

test_that("changes levels, not values", {
  f1 <- factor(c("a"))
  f2 <- factor(c("a"), levels = c("a", "b"))

  expect_equal(lvls_expand(f1, c("a", "b")), f2)
})

test_that("must include all existing levels", {
  f1 <- factor(c("a", "b"))

  expect_error(lvls_expand(f1, c("a", "c")), "include all existing levels")
})

# lvls_revalue ------------------------------------------------------------

test_that("changes values and levels", {
  f1 <- factor(c("a", "b"))
  f2 <- factor(c("b", "a"), levels = c("b", "a"))

  expect_equal(lvls_revalue(f1, c("b", "a")), f2)
})

test_that("can collapse values", {
  f1 <- factor(c("a", "b"))
  f2 <- factor(c("a", "a"))

  expect_equal(lvls_revalue(f1, c("a", "a")), f2)
})

test_that("preserves missing values", {
  f1 <- factor(c("a", NA), exclude = NULL)
  f2 <- lvls_revalue(f1, levels(f1))
  expect_equal(levels(f2), levels(f1))
})

test_that("`new_levels` must be a character", {
  f1 <- factor(c("a", "b"))
  expect_error(lvls_revalue(f1, 1:5), "character vector")
})

test_that("`new_levels` must be same length as existing levels", {
  f1 <- factor(c("a", "b"))

  expect_error(lvls_revalue(f1, c("a")), "same length")
  expect_error(lvls_revalue(f1, c("a", "b", "c")), "same length")
})

context("lvls_reorder")

test_that("changes levels, not values", {
  f1 <- factor(c("a", "b"))
  f2 <- factor(c("a", "b"), levels = c("b", "a"))

  expect_equal(lvls_reorder(f1, 2:1), f2)
})

test_that("idx must be numeric", {
  f <- factor(c("a", "b"))
  expect_error(lvls_reorder(f, "a"), "must be numeric")
})

test_that("must have one integer per level", {
  f <- factor(c("a", "b", "c"))

  expect_error(lvls_reorder(f, c(1, 2)), "one integer for each level")
  expect_error(lvls_reorder(f, c(1, 2, 2)), "one integer for each level")
  expect_error(lvls_reorder(f, c(1, 2.5)), "one integer for each level")
})

test_that("can change ordered status of output", {
  f1 <- factor(letters[1:3])
  f2 <- ordered(f1)

  expect_equal(is.ordered(lvls_reorder(f1, 1:3)), FALSE)
  expect_equal(is.ordered(lvls_reorder(f1, 1:3, ordered = FALSE)), FALSE)
  expect_equal(is.ordered(lvls_reorder(f1, 1:3, ordered = TRUE)), TRUE)

  expect_equal(is.ordered(lvls_reorder(f2, 1:3)), TRUE)
  expect_equal(is.ordered(lvls_reorder(f2, 1:3, ordered = FALSE)), FALSE)
  expect_equal(is.ordered(lvls_reorder(f2, 1:3, ordered = TRUE)), TRUE)
})

#' Reverse order of factor levels
#'
#' This is sometimes useful when plotting a factor.
#'
#' @param f A factor (or character vector).
#' @export
#' @examples
#' f <- factor(c("a", "b", "c"))
#' fct_rev(f)
fct_rev <- function(f) {
  f <- check_factor(f)

  lvls_reorder(f, rev(lvls_seq(f)))
}

context("fct_relevel")

test_that("warns about unknown levels", {
  f1 <- factor(c("a", "b"))

  expect_warning(f2 <- fct_relevel(f1, "d"), "Unknown levels")
  expect_equal(levels(f2), levels(f1))
})

test_that("moves supplied levels to front", {
  f1 <- factor(c("a", "b", "c", "d"))

  f2 <- fct_relevel(f1, "c", "b")
  expect_equal(levels(f2), c("c", "b", "a", "d"))
})

test_that("can moves supplied levels to end", {
  f1 <- factor(c("a", "b", "c", "d"))

  f2 <- fct_relevel(f1, "a", "b", after = 2)
  f3 <- fct_relevel(f1, "a", "b", after = Inf)
  expect_equal(levels(f2), c("c", "d", "a", "b"))
  expect_equal(levels(f3), c("c", "d", "a", "b"))
})

test_that("can relevel with function ", {
  f1 <- fct_rev(factor(c("a", "b")))
  f2a <- fct_relevel(f1, rev)
  f2b <- fct_relevel(f1, ~ rev(.))

  expect_equal(levels(f2a), c("a", "b"))
  expect_equal(levels(f2b), c("a", "b"))
})

test_that("function must return character vector", {
  f <- factor(c("a", "b"))
  expect_error(fct_relevel(f, ~ 1), "character vector")
})

#' Reorder factor levels by hand
#'
#' This is a generalisation of [stats::relevel()] that allows you to move any
#' number of levels to any location.
#'
#' @param .f A factor (or character vector).
#' @param ... Either a function (or formula), or character levels.
#'
#'   A function will be called with the current levels, and the return
#'   value (which must be a character vector) will be used to relevel the
#'   function.
#'
#'   Any levels not mentioned will be left in their existing order, after the
#'   explicitly mentioned levels. Supports tidy dots.
#' @param after Where should the new values be placed?
#' @export
#' @examples
#' f <- factor(c("a", "b", "c", "d"), levels = c("b", "c", "d", "a"))
#' fct_relevel(f)
#' fct_relevel(f, "a")
#' fct_relevel(f, "b", "a")
#'
#' # Move to the third position
#' fct_relevel(f, "a", after = 2)
#'
#' # Relevel to the end
#' fct_relevel(f, "a", after = Inf)
#' fct_relevel(f, "a", after = 3)
#'
#' # Relevel with a function
#' fct_relevel(f, sort)
#' fct_relevel(f, sample)
#' fct_relevel(f, rev)
#'
#' # Using 'Inf' allows you to relevel to the end when the number
#' # of levels is unknown or variable (e.g. vectorised operations)
#' df  <- forcats::gss_cat[, c("rincome", "denom")]
#' lapply(df, levels)
#'
#' df2 <- lapply(df, fct_relevel, "Don't know", after = Inf)
#' lapply(df2, levels)
#'
#' # You'll get a warning if the levels don't exist
#' fct_relevel(f, "e")
fct_relevel <- function(.f, ..., after = 0L) {
  f <- check_factor(.f)

  old_levels <- levels(f)
  if (dots_n(...) == 1L && (is.function(..1) || is_formula(..1))) {
    fun <- as_function(..1)
    first_levels <- fun(old_levels)
    if (!is.character(first_levels)) {
      stop("Re-leveling function must return character vector", call. = FALSE)
    }
  } else {
    first_levels <- chr(...)
  }

  unknown <- setdiff(first_levels, old_levels)
  if (length(unknown) > 0) {
    warning("Unknown levels in `f`: ", paste(unknown, collapse = ", "), call. = FALSE)
    first_levels <- intersect(first_levels, old_levels)
  }

  new_levels <- append(setdiff(old_levels, first_levels), first_levels, after = after)

  lvls_reorder(f, match(new_levels, old_levels))
}

#' A sample of categorical variables from the General Social survey
#'
#' @source Downloaded from \url{https://gssdataexplorer.norc.org/}.
#' @format
#' \describe{
#'  \item{year}{year of survey, 2000--2014}
#'  \item{age}{age. Maximum age truncated to 89.}
#'  \item{marital}{marital status}
#'  \item{race}{race}
#'  \item{rincome}{reported income}
#'  \item{partyid}{party affiliation}
#'  \item{relig}{religion}
#'  \item{denom}{denomination}
#'  \item{tvhours}{hours per day watching tv}
#' }
#' @examples
#' gss_cat
#'
#' fct_count(gss_cat$relig)
#' fct_count(fct_lump(gss_cat$relig))
"gss_cat"

#' Drop unused levels
#'
#' Compared to `base::droplevels()`, does not drop `NA` levels that have values.
#'
#' @param f A factor (or character vector).
#' @param only A character vector restricting the set of levels to be dropped.
#'   If supplied, only levels that have no entries and appear in this vector
#'   will be removed.
#' @export
#' @seealso [fct_expand()] to add additional levels to a factor.
#' @examples
#' f <- factor(c("a", "b"), levels = c("a", "b", "c"))
#' f
#' fct_drop(f)
#'
#' # Set only to restrict which levels to drop
#' fct_drop(f, only = "a")
#' fct_drop(f, only = "c")
fct_drop <- function(f, only) {
  f <- check_factor(f)

  levels <- levels(f)
  count <- table(f)

  to_drop <- levels[count == 0]
  if (!missing(only)) {
    to_drop <- intersect(to_drop, only)
  }

  refactor(f, new_levels = setdiff(levels, to_drop))
}

context("fct_drop")

test_that("doesn't add NA level", {
  f1 <- factor(c("a", NA), levels = c("a", "b"))
  f2 <- fct_drop(f1)

  expect_equal(levels(f2), "a")
})

test_that("can optionally restrict levels to drop", {
  f1 <- factor("a", levels = c("a", "b", "c"))

  expect_equal(levels(fct_drop(f1, only = "b")), c("a", "c"))
  expect_equal(levels(fct_drop(f1, only = "a")), c("a", "b", "c"))
})

test_that("preserves ordered class and attributes", {
  f1 <- ordered(letters[1:2], letters[1:3])
  attr(f1, "x") <- "test"

  f2 <- fct_drop(f1)
  expect_s3_class(f2, "ordered")
  expect_equal(attr(f2, "x"), attr(f1, "x"))
})

#' Automatically relabel factor levels, collapse as necessary
#'
#' @param .f A factor (or character vector).
#' @param .fun A function to be applied to each level. Must accept one
#'   character argument and return a character vector of the same length as
#'   its input.
#'
#'   You can also use `~` to create as shorthand (in the style of purrr).
#'   `~ paste(., "x")` is equivalent to `function(.) paste(., "x")`
#' @param ... Additional arguments to `fun`.
#' @export
#' @examples
#' gss_cat$partyid %>% fct_count()
#' gss_cat$partyid %>% fct_relabel(~ gsub(",", ", ", .x)) %>% fct_count()
#'
#' convert_income <- function(x) {
#'   regex <- "^(?:Lt |)[$]([0-9]+).*$"
#'   is_range <- grepl(regex, x)
#'   num_income <- as.numeric(gsub(regex, "\\1", x[is_range]))
#'   num_income <- trunc(num_income / 5000) * 5000
#'   x[is_range] <- paste0("Gt $", num_income)
#'   x
#' }
#' fct_count(gss_cat$rincome)
#' convert_income(levels(gss_cat$rincome))
#' rincome2 <- fct_relabel(gss_cat$rincome, convert_income)
#' fct_count(rincome2)
fct_relabel <- function(.f, .fun, ...) {
  f <- check_factor(.f)
  .fun <- as_function(.fun)

  old_levels <- levels(f)
  new_levels <- .fun(old_levels, ...)

  lvls_revalue(f, new_levels)
}

context("fct_relabel")

test_that("identity", {
  f1 <- factor(c("a", "b"))

  expect_identical(fct_relabel(f1, identity), f1)
})

test_that("error if not function", {
  f1 <- factor("a")

  expect_error(fct_relabel(f1, 1), "function")
})

test_that("error if level not character", {
  f1 <- factor("a")

  expect_error(fct_relabel(f1, function(x) 1), "character")
})

test_that("error if level has different length", {
  f1 <- factor(letters)

  expect_error(fct_relabel(f1, function(x) x[-1]), "expected 26.*got 25")
})

test_that("total collapse", {
  f1 <- factor(letters)
  new_levels <- function(x) rep("1", length(x))

  expect_identical(fct_relabel(f1, new_levels), factor(new_levels(letters)))
})

test_that("additional arguments", {
  f1 <- factor(letters)

  expect_identical(fct_relabel(f1, paste0, "."), factor(paste0(letters, ".")))
})

test_that("formulas are coerced to functions", {
  f1 <- factor(letters)

  expect_identical(
    fct_relabel(f1, ~paste0(.x, ".")),
    factor(paste0(letters, "."))
  )
})

test_that("string input is coerced to a factor", {
  expect_identical(
    fct_relabel(LETTERS[1:2], .fun=function(x) x),
    factor(LETTERS[1:2])
  )
})

#' Add additional levels to a factor
#'
#' @param f A factor (or character vector).
#' @param ... Additional levels to add to the factor.  Levels that already
#'    exist will be silently ignored.
#' @export
#' @seealso [fct_drop()] to drop unused factor levels.
#' @examples
#' f <- factor(sample(letters[1:3], 20, replace = TRUE))
#' f
#' fct_expand(f, "d", "e", "f")
#' fct_expand(f, letters[1:6])
fct_expand <- function(f, ...) {
  f <- check_factor(f)

  new_levels <- chr(...)
  lvls_expand(f, union(levels(f), new_levels))
}

---
title: "Introduction to forcats"
author: "Emily Robinson"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to forcats}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The goal of the forcats package is to provide a suite of useful tools that solve common problems with factors.
Factors are useful when you have categorical data, variables that have a fixed and known set of values, and when you want to display character vectors in non-alphabetical order.
If you want to learn more, the best place to start is the [chapter on factors](https://r4ds.had.co.nz/factors.html) in R for Data Science.

## Ordering by frequency

```{r message = FALSE}
library(dplyr)
library(ggplot2)
library(forcats)
```

Let's try answering the question, "what are the most common hair colors of star wars characters?" Let's start off by making a bar plot:

```{r initial-plot}
#| fig.alt: >
#|   A bar chart of hair color of starwars characters. The bars are
#|   alphabetically ordered, making it hard to see general patterns.
ggplot(starwars, aes(y = hair_color)) + 
  geom_bar()
```

That's okay, but it would be more helpful the graph was ordered by count.
This is a case of an **unordered** categorical variable where we want it ordered by its frequency.
To do so, we can use the function `fct_infreq()`:

```{r fct-infreq-hair}
#| fig.alt: >
#|   The bar chart of hair color, now ordered so that the least
#|   frequent colours come first and the most frequent colors come last.
#|   This makes it easy to see that the most common hair color is none 
#|   (~35), followed by brown (~18), then black (~12). Surprisingly,
#|   NAs are at the top of the graph, even though there are ~5 NAs and 
#|   other colors have smaller values.
ggplot(starwars, aes(y = fct_infreq(hair_color))) + 
  geom_bar()
```

Note that `fct_infreq()` it automatically puts NA at the top, even though that doesn't have the smallest number of entries.

It's a little surprising that the `NA` bar isn't ordered by frequency.
To understand why we need to make a brief digression to discuss `NA`s in values vs. `NA`s in levels

## `NA`s in levels and values

There are two ways to represent a missing value in a factor:

-   You can include it in the values of the factor; it does not appear in the levels and `is.na()` reports it as missing.
    This is how missing values are encoded in a factor by default:

    ```{r}
    f <- factor(c("x", "y", NA))
    levels(f)
    is.na(f)
    ```

-   You can include it in the levels of the factor and `is.na()` does not report it as missing.
    This requires a little more work to create, because, by default, `factor()` uses `exclude = NA`, meaning that missing values are not included in the levels.
    You can force `NA` to be included by setting `exclude = NULL`:

    ```{r}
    f <- factor(c("x", "y", NA), exclude = NULL)
    levels(f)
    is.na(f)
    ```

`NA`s in the values tend to be best for data analysis, since `is.na()` works as you'd expect.
`NA`s in the levels can be useful for display if you need to control where they appear in a table or a plot.

To fix the issue above, we can use `fct_na_value_to_level()` to convert the `NA` in the value to an NA in the levels.
Then they appear where you'd expect:

```{r}
#| fig.alt: >
#|   The bar chart of hair color, now ordered so that NAs are 
#|   ordered where you'd expect: in between white (4) and black (12).
ggplot(starwars, aes(y = fct_infreq(fct_na_value_to_level(hair_color)))) + 
  geom_bar() + 
  labs(y = "Hair color")
```

(If you need the opposite operation, you can use `fct_na_level_to_value()`.)

## Combining levels

Let's take a look at skin color now:

```{r}
starwars |>
  count(skin_color, sort = TRUE)
```

We see that there's 31 different skin colors - if we want to make a plot this would be way too many to display!
Let's reduce it to only be the top 5.
We can use `fct_lump()` to "lump" all the infrequent colors into one factor, "other." The argument `n` is the number of levels we want to keep.

```{r}
starwars |>
  mutate(skin_color = fct_lump(skin_color, n = 5)) |>
  count(skin_color, sort = TRUE)
```

We could also have used `prop` instead, which keeps all the levels that appear at least `prop` of the time.
For example, let's keep skin colors that at least 10% of the characters have:

```{r}
starwars |>
  mutate(skin_color = fct_lump(skin_color, prop = .1)) |>
  count(skin_color, sort = TRUE)
```

Only light and fair remain; everything else is other.

If you wanted to call it something than "other", you can change it with the argument `other_level`:

```{r}
starwars |>
  mutate(skin_color = fct_lump(skin_color, prop = .1, other_level = "extra")) |>
  count(skin_color, sort = TRUE)
```

What if we wanted to see if the average mass differed by eye color?
We'll only look at the 6 most popular eye colors and remove `NA`s.

```{r fct-lump-mean}
avg_mass_eye_color <- starwars |>
  mutate(eye_color = fct_lump(eye_color, n = 6)) |>
  group_by(eye_color) |>
  summarise(mean_mass = mean(mass, na.rm = TRUE))

avg_mass_eye_color
```

## Ordering by another variable

It looks like people (or at least one person) with orange eyes are definitely heavier!
If we wanted to make a graph, it would be nice if it was ordered by `mean_mass`.
We can do this with `fct_reorder()`, which reorders one variable by another.

```{r fct-reorder}
#| fig-alt: >
#|   A column chart with eye color on the x-axis and mean mass on the 
#|   y-axis. The bars are ordered by mean_mass, so that the tallest bar
#|   (orange eye color with mean mass of ~275) is at the far right.

avg_mass_eye_color |>
  mutate(eye_color = fct_reorder(eye_color, mean_mass)) |>
  ggplot(aes(x = eye_color, y = mean_mass)) + 
  geom_col()
```

## Manually reordering

Let's switch to using another dataset, `gss_cat`, the general social survey.
What is the income distribution among the respondents?

```{r}
gss_cat |>
  count(rincome)
```

Notice that the income levels are in the correct order - they start with the non-answers and then go from highest to lowest.
This is the same order you'd see if you plotted it as a bar chart.
This is not a coincidence.
When you're working with ordinal data, where there is an order, you can have an ordered factor.
You can examine them with the base function `levels()`, which prints them in order:

```{r}
levels(gss_cat$rincome)
```

But what if your factor came in the wrong order?
Let's simulate that by reordering the levels of `rincome` randomly with `fct_shuffle()`:

```{r}
reshuffled_income <- gss_cat$rincome |>
  fct_shuffle()

levels(reshuffled_income)
```

Now if we plotted it, it would show in this order, which is all over the place!
How can we fix this and put it in the right order?

We can use the function `fct_relevel()` when we need to manually reorder our factor levels.
In addition to the factor, you give it a character vector of level names, and specify where you want to move them.
It defaults to moving them to the front, but you can move them after another level with the argument `after`.
If you want to move it to the end, you set `after` equal to `Inf`.

For example, let's say we wanted to move `Lt $1000` and `$1000 to 2999` to the front.
We would write:

```{r}
fct_relevel(reshuffled_income, c("Lt $1000", "$1000 to 2999")) |>
  levels()
```

What if we want to move them to the second and third place?

```{r}
fct_relevel(reshuffled_income, c("Lt $1000", "$1000 to 2999"), after = 1) |>
  levels()
```

---
title: "Introduction to forcats"
author: "Emily Robinson"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to forcats}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The goal of the __forcats__ package is to provide a suite of useful tools that solve common problems with factors. Factors are useful when you have categorical data, variables that have a fixed and known set of values, and when you want to display character vectors in non-alphabetical order. If you want to learn more, the best place to start is the [chapter on factors](https://r4ds.had.co.nz/factors.html) in R for Data Science.  

## Ordering by frequency

```{r message = FALSE}
library(dplyr)
library(ggplot2)
library(forcats)
```

Let's try answering the question, "what are the most common hair colors of star wars characters?" Let's start off by making a bar plot:

```{r initial-plot}
ggplot(starwars, aes(x = hair_color)) + 
  geom_bar() + 
  coord_flip()
```

That's okay, but it would be more helpful the graph was ordered by count. This is a case of an **unordered** categorical variable where we want it ordered by its frequency. To do so, we can use the function `fct_infreq()`:

```{r fct-infreq-hair}
ggplot(starwars, aes(x = fct_infreq(hair_color))) + 
  geom_bar() + 
  coord_flip()
```

Note that `fct_infreq()` it automatically puts NA at the top, even though that doesn't have the smallest number of entries. 

## Combining levels

Let's take a look at skin color now: 

```{r}
starwars %>%
  count(skin_color, sort = TRUE)
```

We see that there's 31 different skin colors - if we want to make a plot this would be way too many to display! Let's reduce it to only be the top 5. We can use `fct_lump()` to "lump" all the infrequent colors into one factor, "other." The argument `n` is the number of levels we want to keep.

```{r}
starwars %>%
  mutate(skin_color = fct_lump(skin_color, n = 5)) %>%
  count(skin_color, sort = TRUE)
```

We could also have used `prop` instead, which keeps all the levels that appear at least `prop` of the time. For example, let's keep skin colors that at least 10% of the characters have: 

```{r}
starwars %>%
  mutate(skin_color = fct_lump(skin_color, prop = .1)) %>%
  count(skin_color, sort = TRUE)
```

Only light and fair remain; everything else is other. 

If you wanted to call it something than "other", you can change it with the argument `other_level`:

```{r}
starwars %>%
  mutate(skin_color = fct_lump(skin_color, prop = .1, other_level = "extra")) %>%
  count(skin_color, sort = TRUE)
```

What if we wanted to see if the average mass differed by eye color? We'll only look at the 6 most popular eye colors and remove `NA`s.  

```{r fct-lump-mean}
avg_mass_eye_color <- starwars %>%
  mutate(eye_color = fct_lump(eye_color, n = 6)) %>%
  group_by(eye_color) %>%
  summarise(mean_mass = mean(mass, na.rm = TRUE))

avg_mass_eye_color
```

## Ordering by another variable

It looks like people (or at least one person) with orange eyes are definitely heavier! If we wanted to make a graph, it would be nice if it was ordered by `mean_mass`. We can do this with `fct_reorder()`, which reorders one variable by another. 

```{r fct-reorder}
avg_mass_eye_color %>%
  mutate(eye_color = fct_reorder(eye_color, mean_mass)) %>%
  ggplot(aes(x = eye_color, y = mean_mass)) + 
  geom_col()
```

## Manually reordering

Let's switch to using another dataset, `gss_cat`, the general social survey. What is the income distribution among the respondents? 

```{r}
gss_cat %>%
  count(rincome)
```

Notice that the income levels are in the correct order - they start with the non-answers and then go from highest to lowest. This is the same order you'd see if you plotted it as a bar chart. This is not a coincidence. When you're working with ordinal data, where there is an order, you can have an ordered factor. You can examine them with the base function `levels()`, which prints them in order:

```{r}
levels(gss_cat$rincome)
```

But what if your factor came in the wrong order? Let's simulate that by reordering the levels of `rincome` randomly with `fct_shuffle()`:

```{r}
reshuffled_income <- gss_cat$rincome %>%
  fct_shuffle()

levels(reshuffled_income)
```

Now if we plotted it, it would show in this order, which is all over the place! How can we fix this and put it in the right order?

We can use the function `fct_relevel()` when we need to manually reorder our factor levels. In addition to the factor, you give it a character vector of level names, and specify where you want to move them. It defaults to moving them to the front, but you can move them after another level with the argument `after`. If you want to move it to the end, you set `after` equal to `Inf`. 

For example, let's say we wanted to move `Lt $1000` and `$1000 to 2999` to the front. We would write:

```{r}
fct_relevel(reshuffled_income, c("Lt $1000", "$1000 to 2999")) %>%
  levels()
```

What if we want to move them to the second and third place? 

```{r}
fct_relevel(reshuffled_income, c("Lt $1000", "$1000 to 2999"), after = 1) %>%
  levels()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recode.R
\name{fct_recode}
\alias{fct_recode}
\title{Change factor levels by hand}
\usage{
fct_recode(.f, ...)
}
\arguments{
\item{.f}{A factor (or character vector).}

\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}> A sequence of named character
vectors where the name gives the new level, and the value gives the old
level. Levels not otherwise mentioned will be left as is. Levels can
be removed by naming them \code{NULL}.}
}
\description{
Change factor levels by hand
}
\examples{
x <- factor(c("apple", "bear", "banana", "dear"))
fct_recode(x, fruit = "apple", fruit = "banana")

# If you make a mistake you'll get a warning
fct_recode(x, fruit = "apple", fruit = "bananana")

# If you name the level NULL it will be removed
fct_recode(x, NULL = "apple", fruit = "banana")

# Wrap the left hand side in quotes if it contains special variables
fct_recode(x, "an apple" = "apple", "a bear" = "bear")

# When passing a named vector to rename levels use !!! to splice
x <- factor(c("apple", "bear", "banana", "dear"))
levels <- c(fruit = "apple", fruit = "banana")
fct_recode(x, !!!levels)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explicit_na.R
\name{fct_explicit_na}
\alias{fct_explicit_na}
\title{Make missing values explicit}
\usage{
fct_explicit_na(f, na_level = "(Missing)")
}
\arguments{
\item{f}{A factor (or character vector).}

\item{na_level}{Level to use for missing values: this is what NAs will be changed to.}
}
\description{
This gives missing values an explicit factor level, ensuring that they
appear in summaries and on plots.
}
\examples{
f1 <- factor(c("a", "a", NA, NA, "a", "b", NA, "c", "a", "c", "b"))
fct_count(f1)

f2 <- fct_explicit_na(f1)
fct_count(f2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapse.R
\name{fct_collapse}
\alias{fct_collapse}
\title{Collapse factor levels into manually defined groups}
\usage{
fct_collapse(.f, ..., other_level = NULL, group_other = "DEPRECATED")
}
\arguments{
\item{.f}{A factor (or character vector).}

\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}> A series of named character vectors. The levels in
each vector will be replaced with the name.}

\item{other_level}{Value of level used for "other" values. Always
placed at end of levels.}

\item{group_other}{Deprecated. Replace all levels not named in \code{...} with "Other"?}
}
\description{
Collapse factor levels into manually defined groups
}
\examples{
fct_count(gss_cat$partyid)

partyid2 <- fct_collapse(gss_cat$partyid,
  missing = c("No answer", "Don't know"),
  other = "Other party",
  rep = c("Strong republican", "Not str republican"),
  ind = c("Ind,near rep", "Independent", "Ind,near dem"),
  dem = c("Not str democrat", "Strong democrat")
)
fct_count(partyid2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rev.R
\name{fct_rev}
\alias{fct_rev}
\title{Reverse order of factor levels}
\usage{
fct_rev(f)
}
\arguments{
\item{f}{A factor (or character vector).}
}
\description{
This is sometimes useful when plotting a factor.
}
\examples{
f <- factor(c("a", "b", "c"))
fct_rev(f)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fct_cross.R
\name{fct_cross}
\alias{fct_cross}
\title{Combine levels from two or more factors to create a new factor}
\usage{
fct_cross(..., sep = ":", keep_empty = FALSE)
}
\arguments{
\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}> Additional factors
or character vectors.}

\item{sep}{A character string to separate the levels}

\item{keep_empty}{If TRUE, keep combinations with no observations as levels}
}
\value{
The new factor
}
\description{
Computes a factor whose levels are all the combinations of the levels of the input factors.
}
\examples{
fruit <- factor(c("apple", "kiwi", "apple", "apple"))
colour <- factor(c("green", "green", "red", "green"))
eaten <- c("yes", "no", "yes", "no")
fct_cross(fruit, colour)
fct_cross(fruit, colour, eaten)
fct_cross(fruit, colour, keep_empty = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/c.R
\name{fct_unify}
\alias{fct_unify}
\title{Unify the levels in a list of factors}
\usage{
fct_unify(fs, levels = lvls_union(fs))
}
\arguments{
\item{fs}{A list of factors}

\item{levels}{Set of levels to apply to every factor. Default to
union of all factor levels}
}
\description{
Unify the levels in a list of factors
}
\examples{
fs <- list(factor("a"), factor("b"), factor(c("a", "b")))
fct_unify(fs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count.R
\name{fct_count}
\alias{fct_count}
\title{Count entries in a factor}
\usage{
fct_count(f, sort = FALSE, prop = FALSE)
}
\arguments{
\item{f}{A factor (or character vector).}

\item{sort}{If \code{TRUE}, sort the result so that the most common values
float to the top.}

\item{prop}{If \code{TRUE}, compute the fraction of marginal table.}
}
\value{
A tibble with columns \code{f}, \code{n} and \code{p}, if prop is \code{TRUE}.
}
\description{
Count entries in a factor
}
\examples{
f <- factor(sample(letters)[rpois(1000, 10)])
table(f)
fct_count(f)
fct_count(f, sort = TRUE)
fct_count(f, sort = TRUE, prop = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expand.R
\name{fct_expand}
\alias{fct_expand}
\title{Add additional levels to a factor}
\usage{
fct_expand(f, ...)
}
\arguments{
\item{f}{A factor (or character vector).}

\item{...}{Additional levels to add to the factor.  Levels that already
exist will be silently ignored.}
}
\description{
Add additional levels to a factor
}
\examples{
f <- factor(sample(letters[1:3], 20, replace = TRUE))
f
fct_expand(f, "d", "e", "f")
fct_expand(f, letters[1:6])
}
\seealso{
\code{\link[=fct_drop]{fct_drop()}} to drop unused factor levels.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lvls.R
\name{lvls}
\alias{lvls}
\alias{lvls_reorder}
\alias{lvls_revalue}
\alias{lvls_expand}
\title{Low-level functions for manipulating levels}
\usage{
lvls_reorder(f, idx, ordered = NA)

lvls_revalue(f, new_levels)

lvls_expand(f, new_levels)
}
\arguments{
\item{f}{A factor (or character vector).}

\item{idx}{A integer index, with one integer for each existing level.}

\item{ordered}{A logical which determines the "ordered" status of the
output factor. \code{NA} preserves the existing status of the factor.}

\item{new_levels}{A character vector of new levels.}
}
\description{
\code{lvls_reorder} leaves values as they are, but changes the order.
\code{lvls_revalue} changes the values of existing levels; there must
be one new level for each old level.
\code{lvls_expand} expands the set of levels; the new levels must
include the old levels.
}
\details{
These functions are less helpful than the higher-level \code{fct_} functions,
but are safer than the very low-level manipulation of levels directly,
because they are more specific, and hence can more carefully check their
arguments.
}
\examples{
f <- factor(c("a", "b", "c"))
lvls_reorder(f, 3:1)
lvls_revalue(f, c("apple", "banana", "carrot"))
lvls_expand(f, c("a", "b", "c", "d"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unique.R
\name{fct_unique}
\alias{fct_unique}
\title{Unique values of a factor}
\usage{
fct_unique(f)
}
\arguments{
\item{f}{A factor.}
}
\description{
Unique values of a factor
}
\examples{
f <- factor(letters[rpois(100, 10)])

unique(f)     # in order of appearance
fct_unique(f) # in order of levels
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shift.R
\name{fct_shift}
\alias{fct_shift}
\title{Shift factor levels to left or right, wrapping around at end}
\usage{
fct_shift(f, n = 1L)
}
\arguments{
\item{f}{A factor.}

\item{n}{Positive values shift to the left; negative values shift to
the right.}
}
\description{
This is useful when the levels of an ordered factor are actually cyclical,
with different conventions on the starting point.
}
\examples{
x <- factor(
  c("Mon", "Tue", "Wed"),
  levels = c("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"),
  ordered = TRUE
)
x
fct_shift(x)
fct_shift(x, 2)
fct_shift(x, -1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/c.R
\name{fct_c}
\alias{fct_c}
\title{Concatenate factors, combining levels}
\usage{
fct_c(...)
}
\arguments{
\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}> Individual
factors. Uses tidy dots, so you can splice in a list of factors
with \verb{!!!}.}
}
\description{
This is a useful way of patching together factors from multiple sources
that really should have the same levels but don't.
}
\examples{
fa <- factor("a")
fb <- factor("b")
fab <- factor(c("a", "b"))

c(fa, fb, fab)
fct_c(fa, fb, fab)

# You can also pass a list of factors with !!!
fs <- list(fa, fb, fab)
fct_c(!!!fs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_factor.R
\name{as_factor}
\alias{as_factor}
\alias{as_factor.factor}
\alias{as_factor.character}
\alias{as_factor.numeric}
\alias{as_factor.logical}
\title{Convert input to a factor}
\usage{
as_factor(x, ...)

\method{as_factor}{factor}(x, ...)

\method{as_factor}{character}(x, ...)

\method{as_factor}{numeric}(x, ...)

\method{as_factor}{logical}(x, ...)
}
\arguments{
\item{x}{Object to coerce to a factor.}

\item{...}{Other arguments passed down to method.}
}
\description{
Compared to base R, when \code{x} is a character, this function creates
levels in the order in which they appear, which will be the same on every
platform. (Base R sorts in the current locale which can vary from place
to place.) When \code{x} is numeric, the ordering is based on the numeric
value and consistent with base R.
}
\details{
This is a generic function.
}
\examples{
# Character object
x <- c("a", "z", "g")
as_factor(x)
as.factor(x)

# Character object containing numbers
y <- c("1.1", "11", "2.2", "22")
as_factor(y)
as.factor(y)

# Numeric object
z <- as.numeric(y)
as_factor(z)
as.factor(z)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lump.R
\name{fct_lump}
\alias{fct_lump}
\alias{fct_lump_min}
\alias{fct_lump_prop}
\alias{fct_lump_n}
\alias{fct_lump_lowfreq}
\title{Lump together factor levels into "other"}
\usage{
fct_lump(
  f,
  n,
  prop,
  w = NULL,
  other_level = "Other",
  ties.method = c("min", "average", "first", "last", "random", "max")
)

fct_lump_min(f, min, w = NULL, other_level = "Other")

fct_lump_prop(f, prop, w = NULL, other_level = "Other")

fct_lump_n(
  f,
  n,
  w = NULL,
  other_level = "Other",
  ties.method = c("min", "average", "first", "last", "random", "max")
)

fct_lump_lowfreq(f, other_level = "Other")
}
\arguments{
\item{f}{A factor (or character vector).}

\item{n}{Positive \code{n} preserves the most common \code{n} values.
Negative \code{n} preserves the least common \code{-n} values.
It there are ties, you will get at least \code{abs(n)} values.}

\item{prop}{Positive \code{prop} lumps values which do not appear at least
\code{prop} of the time. Negative \code{prop} lumps values that
do not appear at most \code{-prop} of the time.}

\item{w}{An optional numeric vector giving weights for frequency of
each value (not level) in f.}

\item{other_level}{Value of level used for "other" values. Always
placed at end of levels.}

\item{ties.method}{A character string specifying how ties are
treated. See \code{\link[=rank]{rank()}} for details.}

\item{min}{Preserve levels that appear at least \code{min} number of times.}
}
\description{
A family for lumping together levels that meet some criteria.
\itemize{
\item \code{fct_lump_min()}: lumps levels that appear fewer than \code{min} times.
\item \code{fct_lump_prop()}: lumps levels that appear in fewer \code{prop * n} times.
\item \code{fct_lump_n()} lumps all levels except for the \code{n} most frequent
(or least frequent if \code{n < 0})
\item \code{fct_lump_lowfreq()} lumps together the least frequent levels, ensuring
that "other" is still the smallest level.
}

\code{fct_lump()} exists primarily for historical reasons, as it automatically
picks between these different methods depending on its arguments.
We no longer recommend that you use it.
}
\examples{
x <- factor(rep(LETTERS[1:9], times = c(40, 10, 5, 27, 1, 1, 1, 1, 1)))
x \%>\% table()
x \%>\% fct_lump_n(3) \%>\% table()
x \%>\% fct_lump_prop(0.10) \%>\% table()
x \%>\% fct_lump_min(5) \%>\% table()
x \%>\% fct_lump_lowfreq() \%>\% table()

x <- factor(letters[rpois(100, 5)])
x
table(x)
table(fct_lump_lowfreq(x))

# Use positive values to collapse the rarest
fct_lump_n(x, n = 3)
fct_lump_prop(x, prop = 0.1)

# Use negative values to collapse the most common
fct_lump_n(x, n = -3)
fct_lump_prop(x, prop = -0.1)

# Use weighted frequencies
w <- c(rep(2, 50), rep(1, 50))
fct_lump_n(x, n = 5, w = w)

# Use ties.method to control how tied factors are collapsed
fct_lump_n(x, n = 6)
fct_lump_n(x, n = 6, ties.method = "max")

# Use fct_lump_min() to lump together all levels with fewer than `n` values
table(fct_lump_min(x, min = 10))
table(fct_lump_min(x, min = 15))
}
\seealso{
\code{\link[=fct_other]{fct_other()}} to convert specified levels to other.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forcats-package.R
\docType{package}
\name{forcats-package}
\alias{forcats}
\alias{forcats-package}
\title{forcats: Tools for Working with Categorical Variables (Factors)}
\description{
\if{html}{\figure{logo.png}{options: align='right' alt='logo' width='120'}}

Helpers for reordering factor levels (including
    moving specified levels to front, ordering by first appearance,
    reversing, and randomly shuffling), and tools for modifying factor
    levels (including collapsing rare levels into other, 'anonymising',
    and manually 'recoding').
}
\seealso{
Useful links:
\itemize{
  \item \url{https://forcats.tidyverse.org}
  \item \url{https://github.com/tidyverse/forcats}
  \item Report bugs at \url{https://github.com/tidyverse/forcats/issues}
}

}
\author{
\strong{Maintainer}: Hadley Wickham \email{hadley@rstudio.com}

Other contributors:
\itemize{
  \item RStudio [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reorder.R
\name{fct_inorder}
\alias{fct_inorder}
\alias{fct_infreq}
\alias{fct_inseq}
\title{Reorder factor levels by first appearance, frequency, or numeric order}
\usage{
fct_inorder(f, ordered = NA)

fct_infreq(f, ordered = NA)

fct_inseq(f, ordered = NA)
}
\arguments{
\item{f}{A factor}

\item{ordered}{A logical which determines the "ordered" status of the
output factor. \code{NA} preserves the existing status of the factor.}
}
\description{
This family of functions changes only the order of the levels.
\itemize{
\item \code{fct_inorder()}: by the order in which they first appear.
\item \code{fct_infreq()}: by number of observations with each level (largest first)
\item \code{fct_inseq()}: by numeric value of level.
}
}
\examples{
f <- factor(c("b", "b", "a", "c", "c", "c"))
f
fct_inorder(f)
fct_infreq(f)

f <- factor(1:3, levels = c("3", "2", "1"))
f
fct_inseq(f)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lvls.R
\name{lvls_union}
\alias{lvls_union}
\title{Find all levels in a list of factors}
\usage{
lvls_union(fs)
}
\arguments{
\item{fs}{A list of factors.}
}
\description{
Find all levels in a list of factors
}
\examples{
fs <- list(factor("a"), factor("b"), factor(c("a", "b")))
lvls_union(fs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/match.R
\name{fct_match}
\alias{fct_match}
\title{Test for presence of levels in a factor}
\usage{
fct_match(f, lvls)
}
\arguments{
\item{f}{A factor (or character vector).}

\item{lvls}{A character vector specifying levels to look for.}
}
\value{
A logical vector
}
\description{
Do any of \code{lvls} occur in \code{f}? Compared to \link{\%in\%}, this function validates
\code{lvls} to ensure that they're actually present in \code{f}. In other words,
\code{x \%in\% "not present"} will return \code{FALSE}, but \code{fct_match(x, "not present")}
will throw an error.
}
\examples{
table(fct_match(gss_cat$marital, c("Married", "Divorced")))

# Compare to \%in\%, misspelled levels throw an error
table(gss_cat$marital \%in\% c("Maried", "Davorced"))
\dontrun{
table(fct_match(gss_cat$marital, c("Maried", "Davorced")))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/other.R
\name{fct_other}
\alias{fct_other}
\title{Replace levels with "other"}
\usage{
fct_other(f, keep, drop, other_level = "Other")
}
\arguments{
\item{f}{A factor (or character vector).}

\item{keep, drop}{Pick one of \code{keep} and \code{drop}:
\itemize{
\item \code{keep} will preserve listed levels, replacing all others with
\code{other_level}.
\item \code{drop} will replace listed levels with \code{other_level}, keeping all
as is.
}}

\item{other_level}{Value of level used for "other" values. Always
placed at end of levels.}
}
\description{
Replace levels with "other"
}
\examples{
x <- factor(rep(LETTERS[1:9], times = c(40, 10, 5, 27, 1, 1, 1, 1, 1)))

fct_other(x, keep = c("A", "B"))
fct_other(x, drop = c("A", "B"))
}
\seealso{
\code{\link[=fct_lump]{fct_lump()}} to automatically convert the rarest (or most
common) levels to "other".
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relevel.R
\name{fct_relevel}
\alias{fct_relevel}
\title{Reorder factor levels by hand}
\usage{
fct_relevel(.f, ..., after = 0L)
}
\arguments{
\item{.f}{A factor (or character vector).}

\item{...}{Either a function (or formula), or character levels.

A function will be called with the current levels, and the return
value (which must be a character vector) will be used to relevel the
function.

Any levels not mentioned will be left in their existing order, after the
explicitly mentioned levels. Supports tidy dots.}

\item{after}{Where should the new values be placed?}
}
\description{
This is a generalisation of \code{\link[stats:relevel]{stats::relevel()}} that allows you to move any
number of levels to any location.
}
\examples{
f <- factor(c("a", "b", "c", "d"), levels = c("b", "c", "d", "a"))
fct_relevel(f)
fct_relevel(f, "a")
fct_relevel(f, "b", "a")

# Move to the third position
fct_relevel(f, "a", after = 2)

# Relevel to the end
fct_relevel(f, "a", after = Inf)
fct_relevel(f, "a", after = 3)

# Relevel with a function
fct_relevel(f, sort)
fct_relevel(f, sample)
fct_relevel(f, rev)

# Using 'Inf' allows you to relevel to the end when the number
# of levels is unknown or variable (e.g. vectorised operations)
df  <- forcats::gss_cat[, c("rincome", "denom")]
lapply(df, levels)

df2 <- lapply(df, fct_relevel, "Don't know", after = Inf)
lapply(df2, levels)

# You'll get a warning if the levels don't exist
fct_relevel(f, "e")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relabel.R
\name{fct_relabel}
\alias{fct_relabel}
\title{Automatically relabel factor levels, collapse as necessary}
\usage{
fct_relabel(.f, .fun, ...)
}
\arguments{
\item{.f}{A factor (or character vector).}

\item{.fun}{A function to be applied to each level. Must accept one
character argument and return a character vector of the same length as
its input.

You can also use \code{~} to create as shorthand (in the style of purrr).
\code{~ paste(., "x")} is equivalent to \code{function(.) paste(., "x")}}

\item{...}{Additional arguments to \code{fun}.}
}
\description{
Automatically relabel factor levels, collapse as necessary
}
\examples{
gss_cat$partyid \%>\% fct_count()
gss_cat$partyid \%>\% fct_relabel(~ gsub(",", ", ", .x)) \%>\% fct_count()

convert_income <- function(x) {
  regex <- "^(?:Lt |)[$]([0-9]+).*$"
  is_range <- grepl(regex, x)
  num_income <- as.numeric(gsub(regex, "\\\\1", x[is_range]))
  num_income <- trunc(num_income / 5000) * 5000
  x[is_range] <- paste0("Gt $", num_income)
  x
}
fct_count(gss_cat$rincome)
convert_income(levels(gss_cat$rincome))
rincome2 <- fct_relabel(gss_cat$rincome, convert_income)
fct_count(rincome2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drop.R
\name{fct_drop}
\alias{fct_drop}
\title{Drop unused levels}
\usage{
fct_drop(f, only)
}
\arguments{
\item{f}{A factor (or character vector).}

\item{only}{A character vector restricting the set of levels to be dropped.
If supplied, only levels that have no entries and appear in this vector
will be removed.}
}
\description{
Compared to \code{base::droplevels()}, does not drop \code{NA} levels that have values.
}
\examples{
f <- factor(c("a", "b"), levels = c("a", "b", "c"))
f
fct_drop(f)

# Set only to restrict which levels to drop
fct_drop(f, only = "a")
fct_drop(f, only = "c")
}
\seealso{
\code{\link[=fct_expand]{fct_expand()}} to add additional levels to a factor.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reorder.R
\name{fct_reorder}
\alias{fct_reorder}
\alias{fct_reorder2}
\alias{last2}
\alias{first2}
\title{Reorder factor levels by sorting along another variable}
\usage{
fct_reorder(.f, .x, .fun = median, ..., .desc = FALSE)

fct_reorder2(.f, .x, .y, .fun = last2, ..., .desc = TRUE)

last2(.x, .y)

first2(.x, .y)
}
\arguments{
\item{.f}{A factor (or character vector).}

\item{.x, .y}{The levels of \code{f} are reordered so that the values
of \code{.fun(.x)} (for \code{fct_reorder()}) and \code{fun(.x, .y)} (for \code{fct_reorder2()})
are in ascending order.}

\item{.fun}{n summary function. It should take one vector for
\code{fct_reorder}, and two vectors for \code{fct_reorder2}, and return a single
value.}

\item{...}{Other arguments passed on to \code{.fun}. A common argument is
\code{na.rm = TRUE}.}

\item{.desc}{Order in descending order? Note the default is different
between \code{fct_reorder} and \code{fct_reorder2}, in order to
match the default ordering of factors in the legend.}
}
\description{
\code{fct_reorder()} is useful for 1d displays where the factor is mapped to
position; \code{fct_reorder2()} for 2d displays where the factor is mapped to
a non-position aesthetic. \code{last2()} and \code{first2()} are helpers for \code{fct_reorder2()};
\code{last2()} finds the last value of \code{y} when sorted by \code{x}; \code{first2()} finds the first value.
}
\examples{
df <- tibble::tribble(
  ~color,     ~a, ~b,
  "blue",      1,  2,
  "green",     6,  2,
  "purple",    3,  3,
  "red",       2,  3,
  "yellow",    5,  1
)
df$color <- factor(df$color)
fct_reorder(df$color, df$a, min)
fct_reorder2(df$color, df$a, df$b)

boxplot(Sepal.Width ~ Species, data = iris)
boxplot(Sepal.Width ~ fct_reorder(Species, Sepal.Width), data = iris)
boxplot(Sepal.Width ~ fct_reorder(Species, Sepal.Width, .desc = TRUE), data = iris)

chks <- subset(ChickWeight, as.integer(Chick) < 10)
chks <- transform(chks, Chick = fct_shuffle(Chick))

if (require("ggplot2")) {
ggplot(chks, aes(Time, weight, colour = Chick)) +
  geom_point() +
  geom_line()

# Note that lines match order in legend
ggplot(chks, aes(Time, weight, colour = fct_reorder2(Chick, Time, weight))) +
  geom_point() +
  geom_line() +
  labs(colour = "Chick")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anon.R
\name{fct_anon}
\alias{fct_anon}
\title{Anonymise factor levels}
\usage{
fct_anon(f, prefix = "")
}
\arguments{
\item{f}{A factor.}

\item{prefix}{A character prefix to insert in front of the random labels.}
}
\description{
Replaces factor levels with arbitrary numeric identifiers. Neither
the values nor the order of the levels are preserved.
}
\examples{
gss_cat$relig \%>\% fct_count()
gss_cat$relig \%>\% fct_anon() \%>\% fct_count()
gss_cat$relig \%>\% fct_anon("X") \%>\% fct_count()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gss_cat}
\alias{gss_cat}
\title{A sample of categorical variables from the General Social survey}
\format{
\describe{
\item{year}{year of survey, 2000--2014}
\item{age}{age. Maximum age truncated to 89.}
\item{marital}{marital status}
\item{race}{race}
\item{rincome}{reported income}
\item{partyid}{party affiliation}
\item{relig}{religion}
\item{denom}{denomination}
\item{tvhours}{hours per day watching tv}
}
}
\source{
Downloaded from \url{https://gssdataexplorer.norc.org/}.
}
\usage{
gss_cat
}
\description{
A sample of categorical variables from the General Social survey
}
\examples{
gss_cat

fct_count(gss_cat$relig)
fct_count(fct_lump(gss_cat$relig))
}
\keyword{datasets}
